@@ -51,6 +51,18 @@
 /* Define to 1 if you have the 'memalign' function. */
 #cmakedefine HAVE_MEMALIGN 1
 
+/* Define to 1 if you have the 'gettimeofday' function. */
+#cmakedefine HAVE_GETTIMEOFDAY 1
+
+/* Define to 1 if you have the 'nanosleep' function. */
+#cmakedefine HAVE_NANOSLEEP 1
+
+/* Define to 1 if you have the 'alphasort' function. */
+#cmakedefine HAVE_ALPHASORT 1
+
+/* Define to 1 if you have the 'scandir' function. */
+#cmakedefine HAVE_SCANDIR 1
+
 
 /* Relative path from bindir to datadir */
 #define BIN2DATADIR "@BIN2DATADIR@"
@@ -58,6 +70,9 @@
 /* Define to 1 to enable DSP 56k emulation for Falcon mode */
 #cmakedefine ENABLE_DSP_EMU 1
 
+/* Define to 1 to enable WINUAE cpu  */
+#cmakedefine ENABLE_WINUAE_CPU 1
+
 /* Define to 1 to use less memory - at the expense of emulation speed */
 #cmakedefine ENABLE_SMALL_MEM 1
 
