@@ -22,6 +22,7 @@ const char DebugDsp_fileid[] = "Hatari debugdsp.c : " __DATE__ " " __TIME__;
 #include "dsp.h"
 #include "evaluate.h"
 #include "memorySnapShot.h"
+#include "profile.h"
 #include "str.h"
 #include "symbols.h"
 
@@ -29,6 +30,7 @@ static Uint16 dsp_disasm_addr;    /* DSP disasm address */
 static Uint16 dsp_memdump_addr;   /* DSP memdump address */
 static char dsp_mem_space = 'P';  /* X, Y, P */
 
+static bool bDspProfiling;        /* Whether profiling is enabled */
 static int nDspActiveCBs = 0;     /* Amount of active conditional breakpoints */
 static int nDspSteps = 0;         /* Amount of steps for DSP single-stepping */
 
@@ -110,14 +112,29 @@ error_msg:
 
 
 /**
- * Check whether given address matches any DSP symbol, if yes,
- * show the matching symbol information.
+ * Check whether given address matches any DSP symbol and whether
+ * there's profiling information available for it.  If yes, show it.
  */
-static void DebugDsp_ShowMatchedSymbol(Uint32 addr)
+static void DebugDsp_ShowAddressInfo(Uint16 addr)
 {
-	const char *symbol = Symbols_GetByDspAddress(addr);
+	Uint32 count, cycles;
+	const char *symbol;
+	bool shown = false;
+
+	symbol = Symbols_GetByDspAddress(addr);
 	if (symbol)
-		fprintf(debugOutput, "%s:\n", symbol);
+	{
+		fprintf(debugOutput, "%s", symbol);
+		shown = true;
+	}
+	if (Profile_DspAddressData(addr, &count, &cycles))
+	{
+		fprintf(debugOutput, "%s%d/%d times/cycles",
+			(shown ? ", " : ""), count, cycles);
+		shown = true;
+	}
+	if (shown)
+		fprintf(debugOutput, ":\n");
 }
 
 
@@ -137,7 +154,7 @@ int DebugDsp_DisAsm(int nArgc, char *psArgs[])
 
 	if (nArgc > 1)
 	{
-		switch (Eval_Range(psArgs[1], &lower, &upper))
+		switch (Eval_Range(psArgs[1], &lower, &upper, true))
 		{
 			case -1:
 				/* invalid value(s) */
@@ -181,7 +198,7 @@ int DebugDsp_DisAsm(int nArgc, char *psArgs[])
 	}
 	printf("DSP disasm 0x%hx-0x%hx:\n", dsp_disasm_addr, dsp_disasm_upper);
 	while (dsp_disasm_addr < dsp_disasm_upper) {
-		DebugDsp_ShowMatchedSymbol(dsp_disasm_addr);
+		DebugDsp_ShowAddressInfo(dsp_disasm_addr);
 		dsp_disasm_addr = DSP_DisasmAddress(dsp_disasm_addr, dsp_disasm_addr);
 	}
 
@@ -224,7 +241,7 @@ int DebugDsp_MemDump(int nArgc, char *psArgs[])
 			fprintf(stderr,"Invalid DSP address space '%c'!\n", space);
 			return DEBUGGER_CMDDONE;
 		}
-		switch (Eval_Range(psArgs[2], &lower, &upper))
+		switch (Eval_Range(psArgs[2], &lower, &upper, true))
 		{
 		case -1:
 			/* invalid value(s) */
@@ -291,7 +308,7 @@ static int DebugDsp_Continue(int nArgc, char *psArgv[])
 
 
 /**
- * DSP wrapper for BreakAddr_Command/BreakPointCount, returns DEBUGGER_END
+ * DSP wrapper for BreakAddr_Command().
  */
 static int DebugDsp_BreakAddr(int nArgc, char *psArgs[])
 {
@@ -300,7 +317,7 @@ static int DebugDsp_BreakAddr(int nArgc, char *psArgs[])
 }
 
 /**
- * DSP wrapper for BreakCond_Command/BreakPointCount, returns DEBUGGER_END
+ * DSP wrapper for BreakCond_Command().
  */
 static int DebugDsp_BreakCond(int nArgc, char *psArgs[])
 {
@@ -308,12 +325,25 @@ static int DebugDsp_BreakCond(int nArgc, char *psArgs[])
 	return DEBUGGER_CMDDONE;
 }
 
+/**
+ * DSP wrapper for Profile_Command().
+ */
+static int DebugDsp_Profile(int nArgc, char *psArgs[])
+{
+	Profile_Command(nArgc, psArgs, true);
+	return DEBUGGER_CMDDONE;
+}
+
 
 /**
  * This function is called after each DSP instruction when debugging is enabled.
  */
 void DebugDsp_Check(void)
 {
+	if (bDspProfiling)
+	{
+		Profile_DspUpdate();
+	}
 	/* TODO: show symbols while disassembling DSP instructions */
 	if (nDspActiveCBs)
 	{
@@ -336,8 +366,10 @@ void DebugDsp_Check(void)
  */
 void DebugDsp_SetDebugging(void)
 {
+	bDspProfiling = Profile_DspStart();
 	nDspActiveCBs = BreakCond_BreakPointCount(true);
-	if (nDspActiveCBs || nDspSteps)
+
+	if (nDspActiveCBs || nDspSteps || bDspProfiling)
 		DSP_SetDebugging(true);
 	else
 		DSP_SetDebugging(false);
@@ -375,6 +407,11 @@ static const dbgcommand_t dspcommands[] =
 	  "load DSP symbols & their addresses",
 	  Symbols_Description,
 	  false },
+	{ DebugDsp_Profile, Profile_Match,
+	  "dspprofile", "dp",
+	  "profile DSP code",
+	  Profile_Description,
+	  false },
 	{ DebugDsp_Register, DebugDsp_MatchRegister,
 	  "dspreg", "dr",
 	  "read/write DSP registers",
@@ -418,4 +455,5 @@ int DebugDsp_Init(const dbgcommand_t **table)
 void DebugDsp_InitSession(void)
 {
 	dsp_disasm_addr = DSP_GetPC();
+	Profile_DspStop();
 }
