@@ -8,13 +8,16 @@
   but since the DMA sound has to be mixed together with the PSG sound and
   the output frequency of the host computer differs from the DMA sound
   frequency, the copy function is a little bit complicated.
-  For reducing copy latency, we set a cycle-interrupt event with
-  CycInt_AddRelativeInterrupt to occur just after a sound frame should be
-  finished. There we update the sound.
   The update function also triggers the ST interrupts (Timer A and MFP-i7)
   which are often used in ST programs for setting a new sound frame after
   the old one has finished.
 
+  To support programs that write into the frame buffer while it's played,
+  we should update dma sound on each video HBL.
+  This is also how it works on a real STE : bytes are read by the DMA
+  at the end of each HBL and stored in a small FIFO (8 bytes) that is sent
+  to the DAC depending on the chosen DMA output freq.
+
   Falcon sound emulation is all taken into account in crossbar.c
 
 
@@ -106,18 +109,21 @@ const char DmaSnd_fileid[] = "Hatari dmaSnd.c : " __DATE__ " " __TIME__;
 
 #define TONE_STEPS 13
 
+#define DMASND_FIFO_SIZE	8			/* 8 bytes : size of the DMA Audio's FIFO, filled on every HBL */
+#define DMASND_FIFO_SIZE_MASK	(DMASND_FIFO_SIZE-1)	/* mask to keep FIFO_pos in 0-7 range */
+
 
 /* Global variables that can be changed/read from other parts of Hatari */
-void DmaSnd_Init_Bass_and_Treble_Tables(void);
 
+static void DmaSnd_Apply_LMC(int nMixBufIdx, int nSamplesToGenerate);
 static void DmaSnd_Set_Tone_Level(int set_bass, int set_treb);
 static float DmaSnd_IIRfilterL(float xn);
 static float DmaSnd_IIRfilterR(float xn);
 static struct first_order_s *DmaSnd_Treble_Shelf(float g, float fc, float Fs);
 static struct first_order_s *DmaSnd_Bass_Shelf(float g, float fc, float Fs);
-static Sint8 DmaSnd_LowPassFilterLeft(Sint8 in);
-static Sint8 DmaSnd_LowPassFilterMono(Sint8 in);
-static Sint8 DmaSnd_LowPassFilterRight(Sint8 in);
+static Sint16 DmaSnd_LowPassFilterLeft(Sint16 in);
+static Sint16 DmaSnd_LowPassFilterRight(Sint16 in);
+static bool DmaSnd_LowPass;
 
 
 Uint16 nDmaSoundControl;                /* Sound control register */
@@ -129,11 +135,22 @@ struct dma_s {
 	Uint16 soundMode;		/* Sound mode register */
 	Uint32 frameStartAddr;		/* Sound frame start */
 	Uint32 frameEndAddr;		/* Sound frame end */
-	Uint32 frameCounter_int;	/* Counter in current sound frame, integer part */
-	Uint32 frameCounter_dec;	/* Counter in current sound frame, decimal part */
-	Uint32 frameLen;		/* Length of the frame */
+	Uint32 frameCounterAddr;	/* Sound frame current address counter */
+
+	/* Internal 8 byte FIFO */
+	Sint8 FIFO[ DMASND_FIFO_SIZE ];
+	Uint16 FIFO_Pos;		/* from 0 to DMASND_FIFO_SIZE-1 */
+	Uint16 FIFO_NbBytes;		/* from 0 to DMASND_FIFO_SIZE */
+
+	Sint16 FrameMono;		/* latest values read from the FIFO */
+	Sint16 FrameLeft;
+	Sint16 FrameRight;
 };
 
+Sint64	frameCounter_float = 0;
+bool	DmaInitSample = false;
+
+
 struct microwire_s {
 	Uint16 data;			/* Microwire Data register */
 	Uint16 mask;			/* Microwire Mask register */
@@ -149,8 +166,9 @@ struct microwire_s {
 struct lmc1992_s {
 	struct first_order_s bass_table[TONE_STEPS];
 	struct first_order_s treb_table[TONE_STEPS];
-	float coef[5];			/* IIR coefs */
-	/*float gain;*/			/* IIR gain*/
+	float coef[5];			/* IIR coefficients */
+	float left_gain;
+	float right_gain;
 };
 
 static struct dma_s dma;
@@ -160,7 +178,6 @@ static struct lmc1992_s lmc1992;
 /* dB = 20log(gain)  :  gain = antilog(dB/20)                                */
 /* Table gain values = (int)(powf(10.0, dB/20.0)*65536.0 + 0.5)  2dB steps   */
 
-
 /* Values for LMC1992 Master volume control (*65536) */
 static const Uint16 LMC1992_Master_Volume_Table[64] =
 {
@@ -190,13 +207,24 @@ static const Sint16 LMC1992_Bass_Treble_Table[16] =
 
 static const int DmaSndSampleRates[4] =
 {
-	CPU_FREQ / 1280,	/* 6258  Hz */
-	CPU_FREQ / 640,		/* 12517 Hz */
-	CPU_FREQ / 320,		/* 25033 Hz */
-	CPU_FREQ / 160		/* 50066 Hz */
+	6258, 12517, 25033, 50066
 };
 
 
+
+/*--------------------------------------------------------------*/
+/* Local functions prototypes					*/
+/*--------------------------------------------------------------*/
+
+static void	DmaSnd_FIFO_Refill(void);
+static Sint8	DmaSnd_FIFO_PullByte(void);
+static void	DmaSnd_FIFO_SetStereo(void);
+
+static int	DmaSnd_DetectSampleRate(void);
+static void	DmaSnd_StartNewFrame(void);
+static inline int DmaSnd_EndOfFrameReached(void);
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Init DMA sound variables.
@@ -212,16 +240,31 @@ void DmaSnd_Init(void)
 void DmaSnd_Reset(bool bCold)
 {
 	nDmaSoundControl = 0;
+	dma.soundMode = 0;
 
-	if (bCold)
+	/* [NP] Set start/end to 0 even on warm reset ? (fix 'Brace' by Diamond Design) */
+	IoMem[0xff8903] = 0;				/* frame start addr = 0 */
+	IoMem[0xff8905] = 0;
+	IoMem[0xff8907] = 0;
+	IoMem[0xff890f] = 0;				/* frame end addr = 0 */
+	IoMem[0xff8911] = 0;
+	IoMem[0xff8913] = 0;
+
+	dma.FIFO_Pos = 0;
+	dma.FIFO_NbBytes = 0;
+	dma.FrameMono = 0;
+	dma.FrameLeft = 0;
+	dma.FrameRight = 0;
+
+	if ( bCold )
 	{
-		dma.soundMode = 3;
-		microwire.masterVolume = 65535;
-		microwire.leftVolume = 65535;
-		microwire.rightVolume = 65535;
+		/* Microwire has no reset signal, it will keep its values on warm reset */
+		microwire.masterVolume = 7;		/* -80 dB ; TOS 1.62 will put 0x28 (ie 65535) = 0 dB (max volume) */
+		microwire.leftVolume = 655;		/* -40 dB ; TOS 1.62 will put 0x14 (ie 65535) = 0 dB (max volume) */
+		microwire.rightVolume = 655;		/* -40 db ; TOS 1.62 will put 0x14 (ie 65535) = 0 dB (max volume) */
 		microwire.mixing = 0;
-		microwire.bass = 6;
-		microwire.treble = 6;
+		microwire.bass = 6;			/* 0 dB (flat) */
+		microwire.treble = 6;			/* 0 dB (flat) */
 	}
 
 	/* Initialise microwire LMC1992 IIR filter parameters */
@@ -244,6 +287,127 @@ void DmaSnd_MemorySnapShot_Capture(bool bSave)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * This function is called on every HBL to ensure the DMA Audio's FIFO
+ * is kept full.
+ * In Hatari, the FIFO is handled like a ring buffer (to avoid memcopying bytes
+ * inside the FIFO when a byte is pushed/pulled).
+ * Note that the DMA fetches words, not bytes, so we read new data only
+ * when 2 bytes or more are missing.
+ * When end of frame is reached, we continue with a new frame if loop mode
+ * is on, else we stop DMA Audio.
+ */
+static void DmaSnd_FIFO_Refill(void)
+{
+	/* If DMA sound is OFF, don't update the FIFO */
+	if ( ( nDmaSoundControl & DMASNDCTRL_PLAY ) == 0)
+		return;
+
+	/* If End Address == Start Address, don't update the FIFO */
+	if (dma.frameEndAddr == dma.frameStartAddr)
+	{
+		DmaSnd_EndOfFrameReached();			/* Stop dma audio if loop mode is off */
+		return;
+	}
+	
+	/* Refill the whole FIFO */
+	while ( DMASND_FIFO_SIZE - dma.FIFO_NbBytes >= 2 )
+	{
+		/* Add one word to the FIFO */
+		LOG_TRACE(TRACE_DMASND, "DMA snd fifo refill adr=%x pos %d nb %d %x %x\n", dma.frameCounterAddr , dma.FIFO_Pos , dma.FIFO_NbBytes ,
+			STRam[ dma.frameCounterAddr ] , STRam[ dma.frameCounterAddr+1 ] );
+
+		dma.FIFO[ ( dma.FIFO_Pos+dma.FIFO_NbBytes+0 ) & DMASND_FIFO_SIZE_MASK ] = (Sint8)STRam[ dma.frameCounterAddr ];	/* add upper byte of the word */
+		dma.FIFO[ ( dma.FIFO_Pos+dma.FIFO_NbBytes+1 ) & DMASND_FIFO_SIZE_MASK ] = (Sint8)STRam[ dma.frameCounterAddr+1 ];	/* add lower byte of the word */
+
+		dma.FIFO_NbBytes += 2;				/* One word more in the FIFO */
+
+		/* Increase current frame address and check if we reached frame's end */
+		dma.frameCounterAddr += 2;
+		if ( dma.frameCounterAddr == dma.frameEndAddr )	/* end of frame reached, should we loop or stop dma ? */
+		{
+			if ( DmaSnd_EndOfFrameReached() )
+				break;				/* Loop mode off, dma audio is now turned off */
+		}
+	}
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Pull one sample/byte from the DMA Audio's FIFO and decrease the number of
+ * remaining bytes.
+ * If the FIFO is empty, return 0 (empty sample)
+ * Note : on a real STE, the 8 bytes FIFO is refilled on each HBL, which gives
+ * a total of 313*8=125326 bytes per sec read by the DMA. As the max freq
+ * is 50066 Hz, the STE can play 100132 bytes per sec in stereo ; so on a real STE
+ * the FIFO can never be empty while DMA is ON.
+ * But on Hatari, if the user chooses an audio's output frequency that is much
+ * lower than the current DMA freq, audio will be updated less frequently than
+ * on each HBL and it could require to process more than DMASND_FIFO_SIZE in one
+ * call to DmaSnd_GenerateSamples(). This is why we allow DmaSnd_FIFO_Refill()
+ * to be called if FIFO is empty but DMA sound is still ON.
+ * This way, sound remains correct even if the user uses very low output freq.
+ */
+static Sint8 DmaSnd_FIFO_PullByte(void)
+{
+	Sint8	sample;
+
+	if ( dma.FIFO_NbBytes == 0 )
+	{
+		DmaSnd_FIFO_Refill();
+		if ( dma.FIFO_NbBytes == 0 )                    /* Refill didn't add any new bytes */
+		{
+			LOG_TRACE(TRACE_DMASND, "DMA snd fifo empty for pull\n" );
+			return 0;
+		}
+	}
+
+
+	LOG_TRACE(TRACE_DMASND, "DMA snd fifo pull pos %d nb %d %02x\n", dma.FIFO_Pos , dma.FIFO_NbBytes , (Uint8)dma.FIFO[ dma.FIFO_Pos ] );
+
+	sample = dma.FIFO[ dma.FIFO_Pos ];			/* Get oldest byte from the FIFO */
+	dma.FIFO_Pos = (dma.FIFO_Pos+1) & DMASND_FIFO_SIZE_MASK;/* Pos to be pulled on next call */
+	dma.FIFO_NbBytes--;					/* One byte less in the FIFO */
+
+	return sample;
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * In case a program switches from mono to stereo, we must ensure that
+ * FIFO_pos is on even boundary to keep Left/Right bytes in the correct
+ * order (Left byte should be on even addresses and Right byte on odd ones).
+ * If this is not the case, we skip one byte.
+ */
+static void DmaSnd_FIFO_SetStereo(void)
+{
+	Uint16	NewPos;
+
+	if ( dma.FIFO_Pos & 1 )
+	{
+		NewPos = (dma.FIFO_Pos+1) & DMASND_FIFO_SIZE_MASK;	/* skip the byte on odd address */
+
+		if ( nDmaSoundControl & DMASNDCTRL_PLAY )	/* print a log if we change while playing */
+			{ LOG_TRACE(TRACE_DMASND, "DMA snd switching to stereo mode while playing mono FIFO_pos %d->%d\n", dma.FIFO_Pos , NewPos ); }
+		else
+			{ LOG_TRACE(TRACE_DMASND, "DMA snd switching to stereo mode FIFO_pos %d->%d\n", dma.FIFO_Pos , NewPos ); }
+
+		dma.FIFO_Pos = NewPos;
+
+		if ( dma.FIFO_NbBytes > 0 )
+			dma.FIFO_NbBytes--;			/* remove one byte if FIFO was not already empty */
+	}
+	
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Returns the frequency corresponding to the 2 lower bits of dma.soundMode
+ */
 static int DmaSnd_DetectSampleRate(void)
 {
 	return DmaSndSampleRates[dma.soundMode & 3];
@@ -253,35 +417,17 @@ static int DmaSnd_DetectSampleRate(void)
 /*-----------------------------------------------------------------------*/
 /**
  * This function is called when a new sound frame is started.
- * It copies the start and end address from the I/O registers, calculates
- * the sample length, etc.
+ * It copies the start and end address from the I/O registers and set
+ * the frame counter addr to the start of this new frame.
  */
 static void DmaSnd_StartNewFrame(void)
 {
-	int nCyclesForFrame;
-
 	dma.frameStartAddr = (IoMem[0xff8903] << 16) | (IoMem[0xff8905] << 8) | (IoMem[0xff8907] & ~1);
 	dma.frameEndAddr = (IoMem[0xff890f] << 16) | (IoMem[0xff8911] << 8) | (IoMem[0xff8913] & ~1);
 
-	dma.frameCounter_int = 0;
-	dma.frameCounter_dec = 0;
-	
-	dma.frameLen = dma.frameEndAddr - dma.frameStartAddr;
+	dma.frameCounterAddr = dma.frameStartAddr;
 
-	if (dma.frameEndAddr <= dma.frameStartAddr)
-	{
-		Log_Printf(LOG_WARN, "DMA snd: Illegal buffer size (from 0x%x to 0x%x)\n",
-		          dma.frameStartAddr, dma.frameEndAddr);
-		dma.frameLen = 0;
-		return;
-	}
-
-	/* To get smooth sound, set an "interrupt" for the end of the frame that
-	 * updates the sound mix buffer. */
-	nCyclesForFrame = dma.frameLen * (CPU_FREQ / DmaSnd_DetectSampleRate());
-	if (!(dma.soundMode & DMASNDMODE_MONO))  /* Is it stereo? */
-		nCyclesForFrame = nCyclesForFrame / 2;
-	CycInt_AddRelativeInterrupt(nCyclesForFrame, INT_CPU_CYCLE, INTERRUPT_DMASOUND);
+	LOG_TRACE(TRACE_DMASND, "DMA snd new frame start=%x end=%x\n", dma.frameStartAddr, dma.frameEndAddr);
 }
 
 
@@ -293,6 +439,8 @@ static void DmaSnd_StartNewFrame(void)
  */
 static inline int DmaSnd_EndOfFrameReached(void)
 {
+	LOG_TRACE(TRACE_DMASND, "DMA snd end of frame\n");
+
 	/* Raise end-of-frame interrupts (MFP-i7 and Time-A) */
 	MFP_InputOnChannel(MFP_TIMER_GPIP7_BIT, MFP_IERA, &MFP_IPRA);
 	if (MFP_TACR == 0x08)       /* Is timer A in Event Count mode? */
@@ -317,44 +465,70 @@ static inline int DmaSnd_EndOfFrameReached(void)
  * Mix DMA sound sample with the normal PSG sound samples.
  * Note: We adjust the volume level of the 8-bit DMA samples to factor
  * 0.75 compared to the PSG sound samples.
+ *
+ * The following formula: -((256*3/4)/4)/4
+ *
+ * Multiply by 256 to convert 8 to 16 bits;
+ * DMA sound is 3/4 level of YM sound;
+ * Divide by 4 to account for MixBuffer[];
+ * Divide by 4 to account for DmaSnd_LowPassFilter;
+ * Multiply DMA sound by -1 because the LMC1992 inverts the signal
+ * ( YM sign is +1 :: -1(op-amp) * -1(Lmc1992) ).
  */
+
+
 void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 {
-	Uint32 FreqRatio;
-	int i, intPart;
+	int i;
 	int nBufIdx;
-	Sint8 *pFrameStart, FrameMono = 0, FrameLeft = 0, FrameRight = 0;
+	Sint8 	MonoByte , LeftByte , RightByte;
 	unsigned n;
+	Sint64 FreqRatio;
 
-	if (!(nDmaSoundControl & DMASNDCTRL_PLAY))
+
+	/* DMA Audio OFF and FIFO empty : process YM2149's output */
+	if ( !(nDmaSoundControl & DMASNDCTRL_PLAY) && ( dma.FIFO_NbBytes == 0 ) )
+	{
+		for (i = 0; i < nSamplesToGenerate; i++)
+		{
+			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+
+			switch (microwire.mixing) {
+				case 1:
+					/* YM2149 */
+					break;
+				default:
+					/* YM2149 - 12dB */
+					MixBuffer[nBufIdx][0] /= 4;
+					MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][0];	
+					break;
+			}
+		}
+
+		/* Apply LMC1992 sound modifications (Bass and Treble) */
+		DmaSnd_Apply_LMC ( nMixBufIdx , nSamplesToGenerate );
+		
 		return;
+	}
 
-	pFrameStart = (Sint8 *)&STRam[dma.frameStartAddr];
 
-	/* Compute ratio between hatari's sound frequency and host computer's sound frequency */
-	FreqRatio = (Uint32)(DmaSnd_DetectSampleRate() / (double)nAudioFrequency * 65536.0 + 0.5);
+	/* DMA Audio ON or FIFO not empty yet */
+
+	/* Compute ratio between DMA's sound frequency and host computer's sound frequency, */
+	/* use << 32 to simulate floating point precision */
+	FreqRatio = ( ((Sint64)DmaSnd_DetectSampleRate()) << 32 ) / nAudioFrequency;
 
 	if (dma.soundMode & DMASNDMODE_MONO)
 	{
 		/* Mono 8-bit */
-
-		n = dma.frameCounter_int;
-
 		for (i = 0; i < nSamplesToGenerate; i++)
 		{
-			/* Is end of DMA buffer reached ? */
-			if (dma.frameCounter_int >= dma.frameLen) {
-				if (DmaSnd_EndOfFrameReached())
-					break;
-				else
-				{	n = dma.frameCounter_int;
-					pFrameStart = (Sint8 *)&STRam[dma.frameStartAddr];
-				}
-			}
-
-			/* Apply anti-aliasing low pass filter ? (mono) */
-			for ( ; n <= dma.frameCounter_int; n++) {
-				FrameMono = DmaSnd_LowPassFilterMono(pFrameStart[n]);
+			if ( DmaInitSample )
+			{
+				MonoByte = DmaSnd_FIFO_PullByte ();
+				dma.FrameMono  = DmaSnd_LowPassFilterLeft( (Sint16)MonoByte );
+				/* No-Click */   DmaSnd_LowPassFilterRight( (Sint16)MonoByte );
+				DmaInitSample = false;
 			}
 
 			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
@@ -362,136 +536,156 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 			switch (microwire.mixing) {
 				case 1:
 					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + (Sint16)FrameMono * -(256*3/4)/4;	
+					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameMono * -((256*3/4)/4)/4;
 					break;
 				case 2:
 					/* DMA sound only */
-					MixBuffer[nBufIdx][0] = (Sint16)FrameMono * -(256*3/4)/4;
+					MixBuffer[nBufIdx][0] = dma.FrameMono * -((256*3/4)/4)/4;
 					break;
 				default:
 					/* DMA and (YM2149 - 12dB) mixing */
 					/* instead of 16462 (-12dB), we approximate by 16384 */
-					MixBuffer[nBufIdx][0] = ((Sint16)FrameMono * -(256*3/4)/4) +
+					MixBuffer[nBufIdx][0] = (dma.FrameMono * -((256*3/4)/4)/4) +
 								(((Sint32)MixBuffer[nBufIdx][0] * 16384)/65536);
 					break;
 			}
-			MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][0];	
 
-			/* Increase ratio pointer to next DMA sample */
-			dma.frameCounter_dec += FreqRatio;
-			if (dma.frameCounter_dec >= 65536) {
-				intPart = dma.frameCounter_dec >> 16;
-				dma.frameCounter_int += intPart;
-				dma.frameCounter_dec -= intPart* 65536;
+			MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][0];		/* right = left */
+
+			/* Increase freq counter */
+			frameCounter_float += FreqRatio;
+			n = frameCounter_float >> 32;				/* number of samples to skip */
+			while ( n > 0 )						/* pull as many bytes from the FIFO as needed */
+			{
+				MonoByte = DmaSnd_FIFO_PullByte ();
+				dma.FrameMono  = DmaSnd_LowPassFilterLeft( (Sint16)MonoByte );
+				/* No-Click */   DmaSnd_LowPassFilterRight( (Sint16)MonoByte );
+				n--;
 			}
+			frameCounter_float &= 0xffffffff;			/* only keep the fractional part */
 		}
 	}
 	else
 	{
 		/* Stereo 8-bit */
-
-		FreqRatio *= 2;
-		n = dma.frameCounter_int & ~1;
-
 		for (i = 0; i < nSamplesToGenerate; i++)
 		{
-			/* Is end of DMA buffer reached ? */
-			if ((dma.frameCounter_int | 1) >= dma.frameLen) {
-				if (DmaSnd_EndOfFrameReached())
-					break;
-				else
-				{	n = dma.frameCounter_int & ~1;
-					pFrameStart = (Sint8 *)&STRam[dma.frameStartAddr];
-				}
-			}
-
-			/* Apply anti-aliasing low pass filter ? (stereo) */
-			for ( ; n <= (dma.frameCounter_int & ~1); n += 2) {
-				FrameLeft  = DmaSnd_LowPassFilterLeft(pFrameStart[n]);
-				FrameRight = DmaSnd_LowPassFilterRight(pFrameStart[n+1]);
+			if ( DmaInitSample )
+			{
+				LeftByte = DmaSnd_FIFO_PullByte ();
+				RightByte = DmaSnd_FIFO_PullByte ();
+				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (Sint16)LeftByte );
+				dma.FrameRight = DmaSnd_LowPassFilterRight( (Sint16)RightByte );
+				DmaInitSample = false;
 			}
 
 			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
-			
+
 			switch (microwire.mixing) {
 				case 1:
 					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + (Sint16)FrameLeft * -(256*3/4)/4;	
-					MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][1] + (Sint16)FrameRight * -(256*3/4)/4;	
+					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
+					MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/4)/4;
 					break;
 				case 2:
 					/* DMA sound only */
-					MixBuffer[nBufIdx][0] = (Sint16)FrameLeft * -(256*3/4)/4;
-					MixBuffer[nBufIdx][1] = (Sint16)FrameRight * -(256*3/4)/4;
+					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
+					MixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/4)/4;
 					break;
 				default:
 					/* DMA and (YM2149 - 12dB) mixing */
 					/* instead of 16462 (-12dB), we approximate by 16384 */
-					MixBuffer[nBufIdx][0] = ((Sint16)FrameLeft * -(256*3/4)/4) +
+					MixBuffer[nBufIdx][0] = (dma.FrameLeft * -((256*3/4)/4)/4) +
 								(((Sint32)MixBuffer[nBufIdx][0] * 16384)/65536);
-					MixBuffer[nBufIdx][1] = ((Sint16)FrameRight * -(256*3/4)/4) +
+					MixBuffer[nBufIdx][1] = (dma.FrameRight * -((256*3/4)/4)/4) +
 								(((Sint32)MixBuffer[nBufIdx][1] * 16384)/65536);
 					break;
 			}
 
-			/* Increase ratio pointer to next DMA sample */
-			dma.frameCounter_dec += FreqRatio;
-			if (dma.frameCounter_dec >= 65536) {
-				intPart = dma.frameCounter_dec >> 16;
-				dma.frameCounter_int += intPart;
-				dma.frameCounter_dec -= intPart* 65536;
+			/* Increase freq counter */
+			frameCounter_float += FreqRatio;
+			n = frameCounter_float >> 32;				/* number of samples to skip */
+			while ( n > 0 )						/* pull as many bytes from the FIFO as needed */
+			{
+				LeftByte = DmaSnd_FIFO_PullByte ();
+				RightByte = DmaSnd_FIFO_PullByte ();
+				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (Sint16)LeftByte );
+				dma.FrameRight = DmaSnd_LowPassFilterRight( (Sint16)RightByte );
+				n--;
 			}
+			frameCounter_float &= 0xffffffff;			/* only keep the fractional part */
 		}
 	}
 
-	/* Apply LMC1992 sound modifications (Bass and Treble) 
-	   The Bass and Treble get samples at nAudioFrequency rate.
-	   The tone control's sampling frequency must be at least 22050 Hz to sound good. 
-	*/
-	for (i = 0; i < nSamplesToGenerate; i++) {
-		nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
-		MixBuffer[nBufIdx][0] = 0.5 + DmaSnd_IIRfilterL(MixBuffer[nBufIdx][0]);
-		MixBuffer[nBufIdx][1] = 0.5 + DmaSnd_IIRfilterR(MixBuffer[nBufIdx][1]);
- 	}
+	/* Apply LMC1992 sound modifications (Bass and Treble) */
+	DmaSnd_Apply_LMC ( nMixBufIdx , nSamplesToGenerate );
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Apply LMC1992 sound modifications (Bass and Treble)
+ * The Bass and Treble get samples at nAudioFrequency rate.
+ * The tone control's sampling frequency must be at least 22050 Hz to sound good.
+ */
+static void DmaSnd_Apply_LMC(int nMixBufIdx, int nSamplesToGenerate)
+{
+	int nBufIdx;
+	int i;
 
 	/* Apply LMC1992 sound modifications (Left, Right and Master Volume) */
 	for (i = 0; i < nSamplesToGenerate; i++) {
 		nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
-		MixBuffer[nBufIdx][0] = (((MixBuffer[nBufIdx][0] * microwire.leftVolume) >> 16) * microwire.masterVolume) >> 16;
-		MixBuffer[nBufIdx][1] = (((MixBuffer[nBufIdx][1] * microwire.rightVolume) >> 16) * microwire.masterVolume) >> 16;
-	}
+		MixBuffer[nBufIdx][0] = DmaSnd_IIRfilterL(MixBuffer[nBufIdx][0]);
+		MixBuffer[nBufIdx][1] = DmaSnd_IIRfilterR(MixBuffer[nBufIdx][1]);
+ 	}
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * DMA sound end of frame "interrupt". Used for updating the sound after
- * a frame has been finished.
+ * STE DMA sound is using an 8 bytes FIFO that is checked and filled on each HBL
+ * (at 50066 Hz 8 bit stereo, the DMA requires approx 6.5 new bytes per HBL)
+ * Calling Sound_Update on each HBL allows to emulate some programs that modify
+ * the data between FrameStart and FrameEnd while DMA sound is ON
+ * (eg the demo 'Mental Hangover' or the game 'Power Up Plus')
+ * We first check if the FIFO needs to be refilled, then we call Sound_Update.
+ * This function should be called from the HBL's handler (in video.c)
  */
-void DmaSnd_InterruptHandler(void)
+void DmaSnd_STE_HBL_Update(void)
 {
-	/* Remove this interrupt from list and re-order */
-	CycInt_AcknowledgeInterrupt();
+	if ( ( ConfigureParams.System.nMachineType != MACHINE_STE )
+	  && ( ConfigureParams.System.nMachineType != MACHINE_MEGA_STE ) )
+		return;
 
-	/* Update sound */
-	Sound_Update();
+
+	/* The DMA starts refilling the FIFO when display is OFF (eg cycle 376 in low res 50 Hz) */
+	DmaSnd_FIFO_Refill ();
+
+	/* If DMA sound is ON or FIFO is not empty, update sound */
+	if  ( (nDmaSoundControl & DMASNDCTRL_PLAY) || ( dma.FIFO_NbBytes > 0 ) )
+		Sound_Update(false);
+
+	/* As long as display is OFF, the DMA will refill the FIFO after playing some samples during the HBL */
+	DmaSnd_FIFO_Refill ();
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Create actual position for frame count registers.
+ * Return current frame counter address (value is always even)
  */
 static Uint32 DmaSnd_GetFrameCount(void)
 {
 	Uint32 nActCount;
 
-	if (nDmaSoundControl & DMASNDCTRL_PLAY)
-		nActCount = dma.frameStartAddr + (int)dma.frameCounter_int;
-	else
-		nActCount = (IoMem[0xff8903] << 16) | (IoMem[0xff8905] << 8) | IoMem[0xff8907];
+	/* Update sound to get the current DMA frame address */
+	Sound_Update(false);
 
-	nActCount &= ~1;
+	if (nDmaSoundControl & DMASNDCTRL_PLAY)
+		nActCount = dma.frameCounterAddr;
+	else
+		nActCount = (IoMem[0xff8903] << 16) | (IoMem[0xff8905] << 8) | (IoMem[0xff8907] & ~1);
 
 	return nActCount;
 }
@@ -509,6 +703,37 @@ void DmaSnd_SoundControl_ReadWord(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Write word to sound control register (0xff8900).
+ */
+void DmaSnd_SoundControl_WriteWord(void)
+{
+	Uint16 nNewSndCtrl;
+
+	LOG_TRACE(TRACE_DMASND, "DMA snd control write: 0x%04x\n", IoMem_ReadWord(0xff8900));
+
+        /* Before starting/stopping DMA sound, create samples up until this point with current values */
+	Sound_Update(false);
+
+	nNewSndCtrl = IoMem_ReadWord(0xff8900) & 3;
+
+	if (!(nDmaSoundControl & DMASNDCTRL_PLAY) && (nNewSndCtrl & DMASNDCTRL_PLAY))
+	{
+		LOG_TRACE(TRACE_DMASND, "DMA snd control write: starting dma sound output\n");
+		DmaInitSample = true;
+		frameCounter_float = 0;
+		DmaSnd_StartNewFrame();
+	}
+	else if ((nDmaSoundControl & DMASNDCTRL_PLAY) && !(nNewSndCtrl & DMASNDCTRL_PLAY))
+	{
+		LOG_TRACE(TRACE_DMASND, "DMA snd control write: stopping dma sound output\n");
+	}
+
+	nDmaSoundControl = nNewSndCtrl;
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Read word from sound frame count high register (0xff8909).
@@ -539,6 +764,65 @@ void DmaSnd_FrameCountLow_ReadByte(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Write bytes to various registers with no action.
+ */
+void DmaSnd_FrameStartHigh_WriteByte(void)
+{
+	LOG_TRACE(TRACE_DMASND, "DMA snd frame start high: 0x%02x at pos %d/%d\n", IoMem_ReadByte(0xff8903) ,
+		dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr );
+}
+
+void DmaSnd_FrameStartMed_WriteByte(void)
+{
+	LOG_TRACE(TRACE_DMASND, "DMA snd frame start med: 0x%02x at pos %d/%d\n", IoMem_ReadByte(0xff8905) ,
+		dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr );
+}
+
+void DmaSnd_FrameStartLow_WriteByte(void)
+{
+	LOG_TRACE(TRACE_DMASND, "DMA snd frame start low: 0x%02x at pos %d/%d\n", IoMem_ReadByte(0xff8907) ,
+		dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr );
+}
+
+void DmaSnd_FrameCountHigh_WriteByte(void)
+{
+	LOG_TRACE(TRACE_DMASND, "DMA snd frame count high: 0x%02x at pos %d/%d\n", IoMem_ReadByte(0xff8909) ,
+		dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr );
+}
+
+void DmaSnd_FrameCountMed_WriteByte(void)
+{
+	LOG_TRACE(TRACE_DMASND, "DMA snd frame count med: 0x%02x at pos %d/%d\n", IoMem_ReadByte(0xff890b) ,
+		dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr );
+}
+
+void DmaSnd_FrameCountLow_WriteByte(void)
+{
+	LOG_TRACE(TRACE_DMASND, "DMA snd frame count low: 0x%02x at pos %d/%d\n", IoMem_ReadByte(0xff890d) ,
+		dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr );
+}
+
+void DmaSnd_FrameEndHigh_WriteByte(void)
+{
+	LOG_TRACE(TRACE_DMASND, "DMA snd frame end high: 0x%02x at pos %d/%d\n", IoMem_ReadByte(0xff890f) ,
+		dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr );
+}
+
+void DmaSnd_FrameEndMed_WriteByte(void)
+{
+	LOG_TRACE(TRACE_DMASND, "DMA snd frame end med: 0x%02x at pos %d/%d\n", IoMem_ReadByte(0xff8911) ,
+		dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr );
+}
+
+void DmaSnd_FrameEndLow_WriteByte(void)
+{
+	LOG_TRACE(TRACE_DMASND, "DMA snd frame end low: 0x%02x at pos %d/%d\n", IoMem_ReadByte(0xff8913) ,
+		dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr );
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Read word from sound mode register (0xff8921).
@@ -557,16 +841,26 @@ void DmaSnd_SoundModeCtrl_ReadByte(void)
  */
 void DmaSnd_SoundModeCtrl_WriteByte(void)
 {
-	LOG_TRACE(TRACE_DMASND, "DMA snd mode write: 0x%02x\n", IoMem_ReadWord(0xff8921));
+	Uint16	SoundModeNew;
 
-	/* STE or TT - hopefully STFM emulation never gets here :)
-	 * We maskout to only hit bits that exist on a real STE
-	 */
-	dma.soundMode = (IoMem_ReadByte(0xff8921) & 0x8f);
-	/* we also write the masked value back into the emulated hw registers so we have a correct value there */
+	SoundModeNew = IoMem_ReadByte(0xff8921);
+
+	LOG_TRACE(TRACE_DMASND, "DMA snd mode write: 0x%02x mode=%s freq=%d\n", SoundModeNew,
+		SoundModeNew & DMASNDMODE_MONO ? "mono" : "stereo" , DmaSndSampleRates[ SoundModeNew & 3 ]);
+
+	/* We maskout to only bits that exist on a real STE */
+	SoundModeNew &= 0x8f;
+
+	/* Are we switching from mono to stereo ? */
+	if ( ( dma.soundMode & DMASNDMODE_MONO ) && ( ( SoundModeNew & DMASNDMODE_MONO ) == 0 ) )
+		DmaSnd_FIFO_SetStereo ();
+
+	dma.soundMode = SoundModeNew;
+	/* We also write the masked value back into the emulated hw registers so we have a correct value there */
 	IoMem_WriteByte(0xff8921, dma.soundMode);
 }
 
+
 /* ---------------------- Microwire / LMC 1992  ---------------------- */
 
 /**
@@ -626,31 +920,41 @@ void DmaSnd_InterruptHandler_Microwire(void)
 		switch ((microwire.data >> 6) & 0x7) {
 			case 0:
 				/* Mixing command */
+				LOG_TRACE ( TRACE_DMASND, "Microwire new mixing=0x%x\n", microwire.data & 0x3 );
 				microwire.mixing = microwire.data & 0x3;
 				break;
 			case 1:
 				/* Bass command */
+				LOG_TRACE ( TRACE_DMASND, "Microwire new bass=0x%x\n", microwire.data & 0xf );
 				microwire.bass = microwire.data & 0xf;
 				DmaSnd_Set_Tone_Level(LMC1992_Bass_Treble_Table[microwire.bass], 
 						      LMC1992_Bass_Treble_Table[microwire.treble]);
 				break;
 			case 2: 
 				/* Treble command */
+				LOG_TRACE ( TRACE_DMASND, "Microwire new trebble=0x%x\n", microwire.data & 0xf );
 				microwire.treble = microwire.data & 0xf;
 				DmaSnd_Set_Tone_Level(LMC1992_Bass_Treble_Table[microwire.bass], 
 						      LMC1992_Bass_Treble_Table[microwire.treble]);
 				break;
 			case 3:
 				/* Master volume command */
+				LOG_TRACE ( TRACE_DMASND, "Microwire new master volume=0x%x\n", microwire.data & 0x3f );
 				microwire.masterVolume = LMC1992_Master_Volume_Table[microwire.data & 0x3f];
+				lmc1992.left_gain = (microwire.leftVolume * (Uint32)microwire.masterVolume) * (1.0/(65536.0*65536.0));
+				lmc1992.right_gain = (microwire.rightVolume * (Uint32)microwire.masterVolume) * (1.0/(65536.0*65536.0));
 				break;
 			case 4:
 				/* Right channel volume */
+				LOG_TRACE ( TRACE_DMASND, "Microwire new right volume=0x%x\n", microwire.data & 0x1f );
 				microwire.rightVolume = LMC1992_LeftRight_Volume_Table[microwire.data & 0x1f];
+				lmc1992.right_gain = (microwire.rightVolume * (Uint32)microwire.masterVolume) * (1.0/(65536.0*65536.0));
 				break;
 			case 5:
 				/* Left channel volume */
+				LOG_TRACE ( TRACE_DMASND, "Microwire new left volume=0x%x\n", microwire.data & 0x1f );
 				microwire.leftVolume = LMC1992_LeftRight_Volume_Table[microwire.data & 0x1f];
+				lmc1992.left_gain = (microwire.leftVolume * (Uint32)microwire.masterVolume) * (1.0/(65536.0*65536.0));
 				break;
 			default:
 				/* Do nothing */
@@ -712,35 +1016,6 @@ void DmaSnd_MicrowireMask_WriteWord(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
-/**
- * Write word to sound control register (0xff8900).
- */
-void DmaSnd_SoundControl_WriteWord(void)
-{
-	Uint16 nNewSndCtrl;
-
-	LOG_TRACE(TRACE_DMASND, "DMA snd control write: 0x%04x\n", IoMem_ReadWord(0xff8900));
-
-	nNewSndCtrl = IoMem_ReadWord(0xff8900) & 3;
-
-	if (!(nDmaSoundControl & DMASNDCTRL_PLAY) && (nNewSndCtrl & DMASNDCTRL_PLAY))
-	{
-		//fprintf(stderr, "Turning on DMA sound emulation.\n");
-		DmaSnd_StartNewFrame();
-	}
-	else if ((nDmaSoundControl & DMASNDCTRL_PLAY) && !(nNewSndCtrl & DMASNDCTRL_PLAY))
-	{
-		//fprintf(stderr, "Turning off DMA sound emulation.\n");
-		/* Create samples up until this point with current values */
-		Sound_Update();
-
-	}
-
-	nDmaSoundControl = nNewSndCtrl;
-}
-
-
 /*-------------------Bass / Treble filter ---------------------------*/
 
 /**
@@ -753,7 +1028,7 @@ static float DmaSnd_IIRfilterL(float xn)
 
 	/* Input coefficients */
 	/* biquad1  Note: 'a' coefficients are subtracted */
-	a  = /*lmc1992.gain * */ xn;		/* a=g*xn;               */
+	a  = lmc1992.left_gain * xn;		/* a=g*xn;               */
 	a -= lmc1992.coef[0] * data[0];		/* a1;  wn-1             */
 	a -= lmc1992.coef[1] * data[1];		/* a2;  wn-2             */
 						/* If coefficient scale  */
@@ -780,7 +1055,7 @@ static float DmaSnd_IIRfilterR(float xn)
 
 	/* Input coefficients */
 	/* biquad1  Note: 'a' coefficients are subtracted */
-	a  = /*lmc1992.gain * */ xn;		/* a=g*xn;               */
+	a  = lmc1992.right_gain * xn;		/* a=g*xn;               */
 	a -= lmc1992.coef[0]*data[0];		/* a1;  wn-1             */
 	a -= lmc1992.coef[1]*data[1];		/* a2;  wn-2             */
 						/* If coefficient scale  */
@@ -796,52 +1071,48 @@ static float DmaSnd_IIRfilterR(float xn)
 	return yn;
 }
 
-/**
- * LowPass Filter Mono
- */
-static Sint8 DmaSnd_LowPassFilterMono(Sint8 in)
-{
-	static int lowPassFilter[2] = { 0, 0 };
-	Sint8 out;
-
-	out = ((lowPassFilter[0]>>1) + (lowPassFilter[1]) + (in>>1)) >> 1;
-	lowPassFilter[0] = lowPassFilter[1];
-	lowPassFilter[1] = in;
-
-	return out;
-}
-
 /**
  * LowPass Filter Left
  */
-static Sint8 DmaSnd_LowPassFilterLeft(Sint8 in)
+static Sint16 DmaSnd_LowPassFilterLeft(Sint16 in)
 {
-	static int lowPassFilter[2] = { 0, 0 };
-	Sint8 out;
+	static	Sint16	lowPassFilter[2] = { 0, 0 };
+	static	Sint16	out = 0;
 
-	out = ((lowPassFilter[0]>>1) + (lowPassFilter[1]) + (in>>1)) >> 1;
-	lowPassFilter[0] = lowPassFilter[1];
-	lowPassFilter[1] = in;
+	if (DmaSnd_LowPass)
+	{
+		out = lowPassFilter[0] + (lowPassFilter[1]<<1) + in;
+		lowPassFilter[0] = lowPassFilter[1];
+		lowPassFilter[1] = in;
 
-	return out;
+		return out; /* Filter Gain = 4 */
+	}else
+	{
+		return in << 2;
+	}
 }
 
 /**
  * LowPass Filter Right
  */
-static Sint8 DmaSnd_LowPassFilterRight(Sint8 in)
+static Sint16 DmaSnd_LowPassFilterRight(Sint16 in)
 {
-	static int lowPassFilter[2] = { 0, 0 };
-	Sint8 out;
+	static	Sint16	lowPassFilter[2] = { 0, 0 };
+	static	Sint16	out = 0;
 
-	out = ((lowPassFilter[0]>>1) + (lowPassFilter[1]) + (in>>1)) >> 1;
-	lowPassFilter[0] = lowPassFilter[1];
-	lowPassFilter[1] = in;
+	if (DmaSnd_LowPass)
+	{
+		out = lowPassFilter[0] + (lowPassFilter[1]<<1) + in;
+		lowPassFilter[0] = lowPassFilter[1];
+		lowPassFilter[1] = in;
 
-	return out;
+		return out; /* Filter Gain = 4 */
+	}else
+	{
+		return in << 2;
+	}
 }
 
-
 /**
  * Set Bass and Treble tone level
  */
@@ -907,7 +1178,7 @@ void DmaSnd_Init_Bass_and_Treble_Tables(void)
 	struct first_order_s *bass;
 	struct first_order_s *treb;
 
-	float  dB, g, fc_bt, fc_tt, Fs;
+	float  dB_adjusted, dB, g, fc_bt, fc_tt, Fs;
 	int    n;
 
 	fc_bt = 118.2763;
@@ -917,6 +1188,26 @@ void DmaSnd_Init_Bass_and_Treble_Tables(void)
 	if ((Fs < 8000.0) || (Fs > 96000.0))
 		Fs = 44100.0;
 
+	if (fc_tt > 0.5*0.8*Fs)
+	{
+		fc_tt = 0.5*0.8*Fs;
+		dB_adjusted = 2.0 * 0.5*0.8*Fs/fc_tt;
+	}else
+	{
+		dB_adjusted = 2.0;
+	}
+
+	for (dB = dB_adjusted*(TONE_STEPS-1)/2, n = TONE_STEPS; n--; dB -= dB_adjusted)
+	{
+		g = powf(10.0, dB/20.0);	/* 12dB to -12dB */
+
+		treb = DmaSnd_Treble_Shelf(g, fc_tt, Fs);
+
+		lmc1992.treb_table[n].a1 = treb->a1;
+		lmc1992.treb_table[n].b0 = treb->b0;
+		lmc1992.treb_table[n].b1 = treb->b1;
+	}
+
 	for (dB = 12.0, n = TONE_STEPS; n--; dB -= 2.0)
 	{
 		g = powf(10.0, dB/20.0);	/* 12dB to -12dB */
@@ -926,14 +1217,18 @@ void DmaSnd_Init_Bass_and_Treble_Tables(void)
 		lmc1992.bass_table[n].a1 = bass->a1;
 		lmc1992.bass_table[n].b0 = bass->b0;
 		lmc1992.bass_table[n].b1 = bass->b1;
-
-		treb = DmaSnd_Treble_Shelf(g, fc_tt, Fs);
-
-		lmc1992.treb_table[n].a1 = treb->a1;
-		lmc1992.treb_table[n].b0 = treb->b0;
-		lmc1992.treb_table[n].b1 = treb->b1;
 	}
 
 	DmaSnd_Set_Tone_Level(LMC1992_Bass_Treble_Table[microwire.bass & 0xf], 
 			      LMC1992_Bass_Treble_Table[microwire.treble & 0xf]);
+
+	/* Initialize IIR Filter Gain and use as a Volume Control */
+	lmc1992.left_gain = (microwire.leftVolume * (Uint32)microwire.masterVolume) * (1.0/(65536.0*65536.0));
+	lmc1992.right_gain = (microwire.rightVolume * (Uint32)microwire.masterVolume) * (1.0/(65536.0*65536.0));
+
+	/* Anti-alias filter is not required when nAudioFrequency == 50066 Hz */
+	if (nAudioFrequency>50000 && nAudioFrequency<50100)
+		DmaSnd_LowPass = false;
+	else
+		DmaSnd_LowPass = true;
 }
