@@ -58,11 +58,6 @@ const char Gemdos_fileid[] = "Hatari gemdos.c : " __DATE__ " " __TIME__;
 /* Maximum supported length of a GEMDOS path: */
 #define MAX_GEMDOS_PATH 256
 
-/* Invalid characters in paths & filenames are replaced by this
- * (valid but very uncommon GEMDOS file name character)
- */
-#define INVALID_CHAR '@'
-
 /* Have we re-directed GemDOS vector to our own routines yet? */
 bool bInitGemDOS;
 
@@ -98,7 +93,7 @@ typedef struct {
 #define  MAX_FILE_HANDLES    32    /* We can allow 32 files open at once */
 
 /*
-   DateTime structure used by TOS call $57 f_dattime
+   DateTime structure used by TOS call $57 f_dtatime
    Changed to fix potential problem with alignment.
 */
 typedef struct {
@@ -141,6 +136,8 @@ static Uint16 nAttrSFirst;  /* File attribute for SFirst/Snext */
 #define S_IROTH 0
 #endif
 
+/* set to 1 if you want to see debug output from pattern matching */
+#define DEBUG_PATTERN_MATCH 0
 
 
 /*-------------------------------------------------------*/
@@ -152,6 +149,7 @@ static bool GemDOS_DateTime2Tos(time_t t, DATETIME *DateTime)
 {
 	struct tm *x;
 
+	/* localtime takes DST into account */
 	x = localtime(&t);
 
 	if (x == NULL)
@@ -207,6 +205,8 @@ static bool GemDOS_SetFileInformation(int Handle, DATETIME *DateTime)
 	timespec.tm_mday = (DateTime->dateword & 0x1F);
 	timespec.tm_mon  = ((DateTime->dateword & 0x1E0) >> 5) - 1;
 	timespec.tm_year = ((DateTime->dateword & 0xFE00) >> 9) + 80;
+	/* check whether DST should be taken into account */
+	timespec.tm_isdst = -1;
 
 	/* set new modification time */
 	timebuf.modtime = mktime(&timespec);
@@ -223,76 +223,6 @@ static bool GemDOS_SetFileInformation(int Handle, DATETIME *DateTime)
 }
 
 
-/**
- * Convert potentially too long host filenames to 8.3 TOS filenames
- * by truncating extension and part before it, replacing invalid
- * GEMDOS file name characters with INVALID_CHAR + upcasing the result.
- * 
- * Matching them from the host file system should first try exact
- * case-insensitive match, and then with a pattern that takes into
- * account the conversion done in here.
- */
-static void Convert2TOSName(char *source, char *dst)
-{
-	char *dot, *tmp, *src;
-	int len;
-
-	src = strdup(source); /* dup so that it can be modified */
-	len = strlen(src);
-
-	/* does filename have an extension? */
-	dot = strrchr(src, '.');
-	if (dot)
-	{
-		/* limit extension to 3 chars */
-		if (src + len - dot > 3)
-			dot[4] = '\0';
-
-		/* if there are extra dots, convert them */
-		for (tmp = src; tmp < dot; tmp++)
-			if (*tmp == '.')
-				*tmp = INVALID_CHAR;
-	}
-
-	/* does name now fit to 8 (+3) chars? */
-	if (len <= 8 || (dot && len <= 12))
-		strcpy(dst, src);
-	else
-	{
-		/* name (still) too long, cut part before extension */
-		strncpy(dst, src, 8);
-		if (dot)
-			strcpy(dst+8, dot);
-		else
-			dst[8] = '\0';
-	}
-	free(src);
-
-	/* replace other invalid chars than '.' in filename */
-	for (tmp = dst; *tmp; tmp++)
-	{
-		if (*tmp < 33 || *tmp > 126)
-			*tmp = INVALID_CHAR;
-		else
-		{
-			switch (*tmp)
-			{
-				case '*':
-				case '/':
-				case ':':
-				case '?':
-				case '\\':
-				case '{':
-				case '}':
-					*tmp = INVALID_CHAR;
-			}
-		}
-	}
-	Str_ToUpper(dst);
-	LOG_TRACE(TRACE_OS_GEMDOS, "host: %s -> GEMDOS: %s\n", source, dst);
-}
-
-
 /*-----------------------------------------------------------------------*/
 /**
  * Convert from FindFirstFile/FindNextFile attribute to GemDOS format
@@ -352,8 +282,11 @@ static int PopulateDTA(char *path, struct dirent *file)
 		return -3;
 
 	/* convert to atari-style uppercase */
-	Convert2TOSName(file->d_name, pDTA->dta_name);
-
+	Str_Filename2TOSname(file->d_name, pDTA->dta_name);
+#if DEBUG_PATTERN_MATCH
+	fprintf(stderr, "GEMDOS: host: %s -> GEMDOS: %s\n",
+		file->d_name, pDTA->dta_name);
+#endif
 	do_put_mem_long(pDTA->dta_size, filestat.st_size);
 	do_put_mem_word(pDTA->dta_time, DateTime.timeword);
 	do_put_mem_word(pDTA->dta_date, DateTime.dateword);
@@ -649,13 +582,13 @@ void GemDOS_InitDrives(void)
 	bool bMultiPartitions;
 
 	/* intialize data for harddrive emulation: */
-	if (!GEMDOS_EMU_ON)
+	if (!emudrives)
 	{
 		emudrives = malloc(MAX_HARDDRIVES * sizeof(EMULATEDDRIVE *));
 		if (!emudrives)
 		{
 			perror("GemDOS_InitDrives");
-			return;
+			abort(); /* fatal */
 		}
 		memset(emudrives, 0, MAX_HARDDRIVES * sizeof(EMULATEDDRIVE *));
 	}
@@ -768,10 +701,12 @@ void GemDOS_MemorySnapShot_Capture(bool bSave)
 	MemorySnapShot_Store(&bEmudrivesAvailable, sizeof(bEmudrivesAvailable));
 	if (bEmudrivesAvailable)
 	{
-		if (!bSave && !emudrives)
+		if (!emudrives)
 		{
-			/* We're loading a memory snapshot, but the emudrives
-			 * structure has not been malloc yet... let's do it now! */
+			/* As memory snapshot contained emulated drive(s),
+			 * but currently there are none allocated yet...
+			 * let's do it now!
+			 */
 			GemDOS_InitDrives();
 		}
 
@@ -783,7 +718,10 @@ void GemDOS_MemorySnapShot_Capture(bool bSave)
 				/* Allocate a dummy drive */
 				emudrives[i] = malloc(sizeof(EMULATEDDRIVE));
 				if (!emudrives[i])
+				{
 					perror("GemDOS_MemorySnapShot_Capture");
+					continue;
+				}
 				memset(emudrives[i], 0, sizeof(EMULATEDDRIVE));
 				bDummyDrive = true;
 			}
@@ -931,8 +869,9 @@ static char* match_host_dir_entry(const char *path, const char *name, bool patte
 	if (!dir)
 		return NULL;
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS match '%s'%s in '%s'", name, pattern?" (pattern)":"", path);
-
+#if DEBUG_PATTERN_MATCH
+	fprintf(stderr, "GEMDOS match '%s'%s in '%s'", name, pattern?" (pattern)":"", path);
+#endif
 	if (pattern)
 	{
 		while ((entry = readdir(dir)))
@@ -956,7 +895,9 @@ static char* match_host_dir_entry(const char *path, const char *name, bool patte
 		}
 	}
 	closedir(dir);
-	LOG_TRACE(TRACE_OS_GEMDOS, " -> '%s'\n", match);
+#if DEBUG_PATTERN_MATCH
+	fprintf(stderr, "-> '%s'\n", match);
+#endif
 	return match;
 }
 
@@ -1014,9 +955,10 @@ static bool add_path_component(char *path, int maxlen, const char *origname, boo
 
 	/* Assume there were invalid characters or that the host file
 	 * was too long to fit into GEMDOS 8+3 filename limits.
-	 * Change name to a pattern that will match such host files
-	 * and try again.
+	 * If that's the case, modify the name to a pattern that
+	 * will match such host files and try again.
 	 */
+	modified = false;
 
 	/* catch potentially invalid characters */
 	for (tmp = name; *tmp; tmp++)
@@ -1423,6 +1365,7 @@ static bool GemDOS_SetDTA(Uint32 Params)
 		pDTA = NULL;
 		Log_Printf(LOG_WARN, "GEMDOS Fsetdta() failed due to invalid DTA address 0x%x\n", nDTA);
 	}
+	/* redirect to TOS */
 	return false;
 }
 
@@ -1444,13 +1387,19 @@ static bool GemDOS_DFree(Uint32 Params)
 	/* is it our drive? */
 	if ((Drive == 0 && CurrentDrive >= 2) || Drive >= 3)
 	{
+		/* Check that write is requested to valid memory area */
+		if (!STMemory_ValidArea(Address, 16))
+		{
+			Log_Printf(LOG_WARN, "GEMDOS Dfree() failed due to invalid RAM range 0x%x+%i\n", Address, 16);
+			Regs[REG_D0] = GEMDOS_ERANGE;
+			return true;
+		}
 		/* FIXME: Report actual free drive space */
-
-		STMemory_WriteLong(Address,  10*2048);           /* free clusters (mock 10 Mb) */
-		STMemory_WriteLong(Address+SIZE_LONG, 50*2048 ); /* total clusters (mock 50 Mb) */
+		STMemory_WriteLong(Address,  16*1024);           /* free clusters (mock 16 Mb) */
+		STMemory_WriteLong(Address+SIZE_LONG, 32*1024 ); /* total clusters (mock 32 Mb) */
 
 		STMemory_WriteLong(Address+SIZE_LONG*2, 512 );   /* bytes per sector */
-		STMemory_WriteLong(Address+SIZE_LONG*3, 1 );     /* sectors per cluster */
+		STMemory_WriteLong(Address+SIZE_LONG*3, 2 );     /* sectors per cluster */
 		return true;
 	}
 	/* redirect to TOS */
@@ -1642,7 +1591,7 @@ static bool GemDOS_FilePathMissing(char *szActualFileName)
 	if (ptr)
 	{
 		*ptr = 0;   /* Strip filename from string */
-		if (!File_DirectoryExists(szActualFileName))
+		if (!File_DirExists(szActualFileName))
 			return true;
 	}
 	return false;
@@ -1780,6 +1729,7 @@ static bool GemDOS_Open(Uint32 Params)
 		{ "rb+", "read/write" }
 	};
 	int Drive, Index, Mode;
+	FILE *AutostartHandle;
 
 	/* Find filename */
 	pszFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params));
@@ -1795,10 +1745,6 @@ static bool GemDOS_Open(Uint32 Params)
 		return false;
 	}
 
-	/* And convert to hard drive filename */
-	GemDOS_CreateHardDriveFileName(Drive, pszFileName,
-	                            szActualFileName, sizeof(szActualFileName));
-
 	/* Find slot to store file handle, as need to return WORD handle for ST  */
 	Index = GemDOS_FindFreeFileHandle();
 	if (Index == -1)
@@ -1819,13 +1765,26 @@ static bool GemDOS_Open(Uint32 Params)
 		ModeStr = Modes[Mode&0x03].mode;
 	}
 
-	/* FIXME: Open file
-	 * - fopen() modes don't allow write-only mode without truncating
-	 *   which would be needed to implement mode 1 (write-only) correctly.
-	 *   Fixing this requires using open() and file descriptors instead
-	 *   of fopen() and FILE* pointers, but Windows doesn't support that.
-	 */
-	FileHandles[Index].FileHandle =  fopen(szActualFileName, ModeStr);
+	
+	if ((AutostartHandle = TOS_AutoStartOpen(pszFileName)))
+	{
+		strcpy(szActualFileName, pszFileName);
+		FileHandles[Index].FileHandle = AutostartHandle;
+	}
+	else
+	{
+		/* Convert to hard drive filename */
+		GemDOS_CreateHardDriveFileName(Drive, pszFileName,
+			szActualFileName, sizeof(szActualFileName));
+
+		/* FIXME: Open file
+		 * - fopen() modes don't allow write-only mode without truncating
+		 *   which would be needed to implement mode 1 (write-only) correctly.
+		 *   Fixing this requires using open() and file descriptors instead
+		 *   of fopen() and FILE* pointers, but Windows doesn't support that.
+		 */
+		FileHandles[Index].FileHandle =  fopen(szActualFileName, ModeStr);
+	}
 
 	if (FileHandles[Index].FileHandle != NULL)
 	{
@@ -1885,7 +1844,10 @@ static bool GemDOS_Close(Uint32 Params)
 	}
 	
 	/* Close file and free up handle table */
-	fclose(FileHandles[Handle].FileHandle);
+	if (!TOS_AutoStartClose(FileHandles[Handle].FileHandle))
+	{
+		fclose(FileHandles[Handle].FileHandle);
+	}
 	FileHandles[Handle].bUsed = false;
 
 	/* Return no error */
@@ -1955,7 +1917,7 @@ static bool GemDOS_Read(Uint32 Params)
 	/* Check that read is to valid memory area */
 	if (!STMemory_ValidArea(Addr, Size))
 	{
-		Log_Printf(LOG_TODO, "GEMDOS Fread() failed due to invalid RAM range 0x%x+%i\n", Addr, Size);
+		Log_Printf(LOG_WARN, "GEMDOS Fread() failed due to invalid RAM range 0x%x+%i\n", Addr, Size);
 		Regs[REG_D0] = GEMDOS_ERANGE;
 		return true;
 	}
@@ -2010,7 +1972,7 @@ static bool GemDOS_Write(Uint32 Params)
 	/* Check that write is from valid memory area */
 	if (!STMemory_ValidArea(Addr, Size))
 	{
-		Log_Printf(LOG_TODO, "GEMDOS Fwrite() failed due to invalid RAM range 0x%x+%i\n", Addr, Size);
+		Log_Printf(LOG_WARN, "GEMDOS Fwrite() failed due to invalid RAM range 0x%x+%i\n", Addr, Size);
 		Regs[REG_D0] = GEMDOS_ERANGE;
 		return true;
 	}
@@ -2260,7 +2222,7 @@ static bool GemDOS_Fattrib(Uint32 Params)
  * GEMDOS Get Directory
  * Call 0x47
  */
-static int GemDOS_GetDir(Uint32 Params)
+static bool GemDOS_GetDir(Uint32 Params)
 {
 	Uint32 Address;
 	Uint16 Drive;
@@ -2293,12 +2255,19 @@ static int GemDOS_GetDir(Uint32 Params)
 		// convert it to ST path (DOS)
 		File_CleanFileName(path);
 		len = strlen(path);
+		/* Check that write is requested to valid memory area */
+		if (!STMemory_ValidArea(Address, len))
+		{
+			Log_Printf(LOG_WARN, "GEMDOS Dgetpath() failed due to invalid RAM range 0x%x+%i\n", Address, len);
+			Regs[REG_D0] = GEMDOS_ERANGE;
+			return true;
+		}
 		for (i = 0; i <= len; i++)
 		{
 			c = path[i];
 			STMemory_WriteByte(Address+i, (c==PATHSEP ? '\\' : c) );
 		}
-		LOG_TRACE(TRACE_OS_GEMDOS, "GemDOS_GetDir (%d) = %s\n", Drive, path );
+		LOG_TRACE(TRACE_OS_GEMDOS, "-> '%s'\n", (char *)STRAM_ADDR(Address));
 
 		Regs[REG_D0] = GEMDOS_EOK;          /* OK */
 
@@ -2597,109 +2566,6 @@ static bool GemDOS_Rename(Uint32 Params)
 }
 
 
-#if ENABLE_TRACING
-/*-----------------------------------------------------------------------*/
-/**
- * Map GEMDOS call opcodes to their names
- */
-static const char* GemDOS_Opcode2Name(Uint16 opcode)
-{
-	static const char* names[] = {
-		"Pterm0",
-		"Cconin",
-		"Cconout",
-		"Cauxin",
-		"Cauxout",
-		"Cprnout",
-		"Crawio",
-		"Crawcin",
-		"Cnecin",
-		"Cconws",
-		"Cconrs",
-		"Cconis",
-		"", /* 0C */
-		"", /* 0D */
-		"Dsetdrv",
-		"", /* 0F */
-		"Cconos",
-		"Cprnos",
-		"Cauxis",
-		"Cauxos",
-		"Maddalt",
-		"", /* 15 */
-		"", /* 16 */
-		"", /* 17 */
-		"", /* 18 */
-		"Dgetdrv",
-		"Fsetdta",
-		"", /* 1B */
-		"", /* 1C */
-		"", /* 1D */
-		"", /* 1E */
-		"", /* 1F */
-		"Super",
-		"", /* 21 */
-		"", /* 22 */
-		"", /* 23 */
-		"", /* 24 */
-		"", /* 25 */
-		"", /* 26 */
-		"", /* 27 */
-		"", /* 28 */
-		"", /* 29 */
-		"Tgetdate",
-		"Tsetdate",
-		"Tgettime",
-		"Tsettime",
-		"", /* 2E */
-		"Fgetdta",
-		"Sversion",
-		"Ptermres",
-		"", /* 32 */
-		"", /* 33 */
-		"", /* 34 */
-		"", /* 35 */
-		"Dfree",
-		"", /* 37 */
-		"", /* 38 */
-		"Dcreate",
-		"Ddelete",
-		"Dsetpath",
-		"Fcreate",
-		"Fopen",
-		"Fclose",
-		"Fread",
-		"Fwrite",
-		"Fdelete",
-		"Fseek",
-		"Fattrib",
-		"Mxalloc",
-		"Fdup",
-		"Fforce",
-		"Dgetpath",
-		"Malloc",
-		"Mfree",
-		"Mshrink",
-		"Pexec",
-		"Pterm",
-		"", /* 4D */
-		"Fsfirst",
-		"Fsnext",
-		"", /* 50 */
-		"", /* 51 */
-		"", /* 52 */
-		"", /* 53 */
-		"", /* 54 */
-		"", /* 55 */
-		"Frename",
-		"Fdatime"
-	};
-	if (opcode < ARRAYSIZE(names))
-		return names[opcode];
-	return "MiNT call?";
-}
-#endif
-
 /*-----------------------------------------------------------------------*/
 /**
  * GEMDOS GSDToF
@@ -2748,9 +2614,18 @@ static bool GemDOS_GSDToF(Uint32 Params)
 
 	if (GemDOS_GetFileInformation(Handle, &DateTime) == true)
 	{
-		STMemory_WriteWord(pBuffer, DateTime.timeword);
-		STMemory_WriteWord(pBuffer+SIZE_WORD, DateTime.dateword);
-		Regs[REG_D0] = GEMDOS_EOK;
+		/* Check that write is requested to valid memory area */
+		if (STMemory_ValidArea(pBuffer, 4))
+		{
+			STMemory_WriteWord(pBuffer, DateTime.timeword);
+			STMemory_WriteWord(pBuffer+SIZE_WORD, DateTime.dateword);
+			Regs[REG_D0] = GEMDOS_EOK;
+		}
+		else
+		{
+			Log_Printf(LOG_WARN, "GEMDOS Fdatime() failed due to invalid RAM range 0x%x+%i\n", pBuffer, 4);
+			Regs[REG_D0] = GEMDOS_ERANGE;
+		}
 	}
 	else
 	{
@@ -2760,6 +2635,189 @@ static bool GemDOS_GSDToF(Uint32 Params)
 }
 
 
+#if ENABLE_TRACING
+/*-----------------------------------------------------------------------*/
+/**
+ * Map GEMDOS call opcodes to their names
+ */
+static const char* GemDOS_Opcode2Name(Uint16 opcode)
+{
+	static const char* names[] = {
+		"Pterm0",
+		"Cconin",
+		"Cconout",
+		"Cauxin",
+		"Cauxout",
+		"Cprnout",
+		"Crawio",
+		"Crawcin",
+		"Cnecin",
+		"Cconws",
+		"Cconrs",
+		"Cconis",
+		"-", /* 0C */
+		"-", /* 0D */
+		"Dsetdrv",
+		"-", /* 0F */
+		"Cconos",
+		"Cprnos",
+		"Cauxis",
+		"Cauxos",
+		"Maddalt",
+		"-", /* 15 */
+		"-", /* 16 */
+		"-", /* 17 */
+		"-", /* 18 */
+		"Dgetdrv",
+		"Fsetdta",
+		"-", /* 1B */
+		"-", /* 1C */
+		"-", /* 1D */
+		"-", /* 1E */
+		"-", /* 1F */
+		"Super",
+		"-", /* 21 */
+		"-", /* 22 */
+		"-", /* 23 */
+		"-", /* 24 */
+		"-", /* 25 */
+		"-", /* 26 */
+		"-", /* 27 */
+		"-", /* 28 */
+		"-", /* 29 */
+		"Tgetdate",
+		"Tsetdate",
+		"Tgettime",
+		"Tsettime",
+		"-", /* 2E */
+		"Fgetdta",
+		"Sversion",
+		"Ptermres",
+		"-", /* 32 */
+		"-", /* 33 */
+		"-", /* 34 */
+		"-", /* 35 */
+		"Dfree",
+		"-", /* 37 */
+		"-", /* 38 */
+		"Dcreate",
+		"Ddelete",
+		"Dsetpath",
+		"Fcreate",
+		"Fopen",
+		"Fclose",
+		"Fread",
+		"Fwrite",
+		"Fdelete",
+		"Fseek",
+		"Fattrib",
+		"Mxalloc",
+		"Fdup",
+		"Fforce",
+		"Dgetpath",
+		"Malloc",
+		"Mfree",
+		"Mshrink",
+		"Pexec",
+		"Pterm",
+		"-", /* 4D */
+		"Fsfirst",
+		"Fsnext",
+		"-", /* 50 */
+		"-", /* 51 */
+		"-", /* 52 */
+		"-", /* 53 */
+		"-", /* 54 */
+		"-", /* 55 */
+		"Frename",
+		"Fdatime"
+	};
+	if (opcode < ARRAYSIZE(names))
+		return names[opcode];
+	return "MiNT?";
+}
+
+/**
+ * If bShowOpcodes is true, show GEMDOS call opcode/function name table,
+ * otherwise GEMDOS HDD emulation information.
+ */
+void GemDOS_Info(Uint32 bShowOpcodes)
+{
+	int i, used;
+
+	if (bShowOpcodes)
+	{
+		Uint16 opcode;
+		for (opcode = 0; opcode < 0x5A; )
+		{
+			fprintf(stderr, "%02x %-9s",
+				opcode, GemDOS_Opcode2Name(opcode));
+			if (++opcode % 6 == 0)
+				fputs("\n", stderr);
+		}
+		return;
+	}
+
+	if (!GEMDOS_EMU_ON)
+	{
+		fputs("GEMDOS HDD emulation isn't enabled!\n", stderr);
+		return;
+	}
+
+	fputs("GEMDOS HDD emulation drives:\n", stderr);
+	for(i = 0; i<MAX_HARDDRIVES; i++)
+	{
+		if (!emudrives[i])
+			continue;
+		fprintf(stderr, "- %c: %s\n",
+			'A' + emudrives[i]->drive_number,
+			emudrives[i]->hd_emulation_dir);
+	}
+
+	fputs("\nInternal Fsfirst() DTAs:\n", stderr);
+	for(used = i = 0; i < MAX_DTAS_FILES; i++)
+	{
+		int j, centry, entries;
+
+		if (!InternalDTAs[i].bUsed)
+			continue;
+
+		fprintf(stderr, "+ %d: %s\n", i, InternalDTAs[i].path);
+		
+		centry = InternalDTAs[i].centry;
+		entries = InternalDTAs[i].nentries;
+		for (j = 0; j < entries; j++)
+		{
+			fprintf(stderr, "  - %d: %s%s\n",
+				j, InternalDTAs[i].found[j]->d_name,
+				j == centry ? " *" : "");
+		}
+		fprintf(stderr, "  Fsnext entry = %d.\n", centry);
+		used++;
+	}
+	if (!used)
+		fputs("- None in use.\n", stderr);
+
+	fputs("\nOpen GEMDOS HDD file handles:\n", stderr);
+	for (used = i = 0; i < MAX_FILE_HANDLES; i++)
+	{
+		if (!FileHandles[i].bUsed)
+			continue;
+		fprintf(stderr, "- %d: %s\n", i, FileHandles[i].szActualName);
+		used++;
+	}
+	if (!used)
+		fputs("- None.\n", stderr);
+}
+
+#else /* !ENABLE_TRACING */
+void GemDOS_Info(Uint32 bShowOpcodes)
+{
+	fputs("Hatari isn't configured with ENABLE_TRACING\n", stderr);
+}
+#endif /* !ENABLE_TRACING */
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Run GEMDos call, and re-direct if need to. Used to handle hard disk emulation etc...
@@ -2773,7 +2831,7 @@ void GemDOS_OpCode(void)
 {
 	Uint16 GemDOSCall, CallingSReg;
 	Uint32 Params;
-	short RunOld;
+	int Finished;
 	Uint16 SR;
 
 	SR = M68000_GetSR();
@@ -2790,7 +2848,7 @@ void GemDOS_OpCode(void)
 	}
 
 	/* Default to run TOS GemDos (SR_NEG run Gemdos, SR_ZERO already done, SR_OVERFLOW run own 'Pexec' */
-	RunOld = true;
+	Finished = false;
 	SR &= SR_CLEAR_OVERFLOW;
 	SR &= SR_CLEAR_ZERO;
 	SR |= SR_NEG;
@@ -2804,130 +2862,105 @@ void GemDOS_OpCode(void)
 	{
 	 /*
 	 case 0x3:
-		if (GemDOS_Cauxin(Params))
-			RunOld = false;
+		Finished = GemDOS_Cauxin(Params);
 		break;
 	 */
 	 /*
 	 case 0x4:
-		if (GemDOS_Cauxout(Params))
-			RunOld = false;
+		Finished = GemDOS_Cauxout(Params);
 		break;
 	 */
 	 /* direct printing via GEMDOS */
 	 /*
 	 case 0x5:
-		if (GemDOS_Cprnout(Params))
-			RunOld = false;
+		Finished = GemDOS_Cprnout(Params);
 		break;
 	 */
 	 case 0xe:
-		if (GemDOS_SetDrv(Params))
-			RunOld = false;
+		Finished = GemDOS_SetDrv(Params);
 		break;
 	 /* Printer status  */
 	 /*
 	 case 0x11:
-		if (GemDOS_Cprnos(Params))
-			RunOld = false;
+		Finished = GemDOS_Cprnos(Params);
 		break;
 	 */
 	 /*
 	 case 0x12:
-		if (GemDOS_Cauxis(Params))
-			RunOld = false;
+		Finished = GemDOS_Cauxis(Params);
 		break;
 	 */
 	 /*
 	 case 0x13:
-		if (GemDOS_Cauxos(Params))
-			RunOld = false;
+		Finished = GemDOS_Cauxos(Params);
 		break;
 	 */
 	 case 0x1a:
-		if (GemDOS_SetDTA(Params))
-			RunOld = false;
+		Finished = GemDOS_SetDTA(Params);
 		break;
 	 case 0x36:
-		if (GemDOS_DFree(Params))
-			RunOld = false;
+		Finished = GemDOS_DFree(Params);
 		break;
 	 case 0x39:
-		if (GemDOS_MkDir(Params))
-			RunOld = false;
+		Finished = GemDOS_MkDir(Params);
 		break;
 	 case 0x3a:
-		if (GemDOS_RmDir(Params))
-			RunOld = false;
+		Finished = GemDOS_RmDir(Params);
 		break;
 	 case 0x3b:
-		if (GemDOS_ChDir(Params))
-			RunOld = false;
+		Finished = GemDOS_ChDir(Params);
 		break;
 	 case 0x3c:
-		if (GemDOS_Create(Params))
-			RunOld = false;
+		Finished = GemDOS_Create(Params);
 		break;
 	 case 0x3d:
-		if (GemDOS_Open(Params))
-			RunOld = false;
+		Finished = GemDOS_Open(Params);
 		break;
 	 case 0x3e:
-		if (GemDOS_Close(Params))
-			RunOld = false;
+		Finished = GemDOS_Close(Params);
 		break;
 	 case 0x3f:
-		if (GemDOS_Read(Params))
-			RunOld = false;
+		Finished = GemDOS_Read(Params);
 		break;
 	 case 0x40:
-		if (GemDOS_Write(Params))
-			RunOld = false;
+		Finished = GemDOS_Write(Params);
 		break;
 	 case 0x41:
-		if (GemDOS_FDelete(Params))
-			RunOld = false;
+		Finished = GemDOS_FDelete(Params);
 		break;
 	 case 0x42:
-		if (GemDOS_LSeek(Params))
-			RunOld = false;
+		Finished = GemDOS_LSeek(Params);
 		break;
 	 case 0x43:
-		if (GemDOS_Fattrib(Params))
-			RunOld = false;
+		Finished = GemDOS_Fattrib(Params);
 		break;
 	 case 0x47:
-		if (GemDOS_GetDir(Params))
-			RunOld = false;
+		Finished = GemDOS_GetDir(Params);
 		break;
 	 case 0x4b:
-		if (GemDOS_Pexec(Params) == CALL_PEXEC_ROUTINE)
-			RunOld = CALL_PEXEC_ROUTINE;
+		/* Either false or CALL_PEXEC_ROUTINE */
+		Finished = GemDOS_Pexec(Params);
 		break;
 	 case 0x4e:
-		if (GemDOS_SFirst(Params))
-			RunOld = false;
+		Finished = GemDOS_SFirst(Params);
 		break;
 	 case 0x4f:
-		if (GemDOS_SNext())
-			RunOld = false;
+		Finished = GemDOS_SNext();
 		break;
 	 case 0x56:
-		if (GemDOS_Rename(Params))
-			RunOld = false;
+		Finished = GemDOS_Rename(Params);
 		break;
 	 case 0x57:
-		if (GemDOS_GSDToF(Params))
-			RunOld = false;
+		Finished = GemDOS_GSDToF(Params);
 		break;
 	 default:
 		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS call 0x%X (%s)\n",
 			  GemDOSCall, GemDOS_Opcode2Name(GemDOSCall));
 	}
 
-	switch(RunOld)
+	switch(Finished)
 	{
-	 case false:
+	 case true:
 		/* skip over branch to pexec to RTE */
 		SR |= SR_ZERO;
 		/* visualize GemDOS emu HD access? */
