@@ -516,7 +516,7 @@ static const int nSoundFreqs[] =
 - (void)initKeysDropDown:(NSPopUpButton*)dropDown
 {
 	[dropDown removeAllItems];
-	int i;
+	unsigned int i;
 	for (i = 0; i < Preferences_cKeysForJoysticks; i++)
 	{
 		SDLKey key = Preferences_KeysForJoysticks[i];
@@ -611,7 +611,7 @@ static const int nSoundFreqs[] =
 	IMPORT_SWITCH(autoInsertB, ConfigureParams.DiskImage.bAutoInsertDiskB);
     IMPORT_SWITCH(blitter, ConfigureParams.System.bBlitter);
 	IMPORT_SWITCH(bootFromHD, ConfigureParams.HardDisk.bBootFromHardDisk);	
-    IMPORT_SWITCH(captureOnChange, ConfigureParams.Screen.bCaptureChange);
+    IMPORT_SWITCH(captureOnChange, ConfigureParams.Screen.bCrop);
     IMPORT_TEXTFIELD(cartridgeImage, ConfigureParams.Rom.szCartridgeImageFileName);
     IMPORT_RADIO(colorDepth, ConfigureParams.Screen.nVdiColors);
     IMPORT_SWITCH(compatibleCpu, ConfigureParams.System.bCompatibleCpu);
@@ -822,7 +822,7 @@ static const int nSoundFreqs[] =
 	EXPORT_SWITCH(autoInsertB, ConfigureParams.DiskImage.bAutoInsertDiskB);
     EXPORT_SWITCH(blitter, ConfigureParams.System.bBlitter);
 	EXPORT_SWITCH(bootFromHD, ConfigureParams.HardDisk.bBootFromHardDisk);
-    EXPORT_SWITCH(captureOnChange, ConfigureParams.Screen.bCaptureChange);
+    EXPORT_SWITCH(captureOnChange, ConfigureParams.Screen.bCrop);
     EXPORT_TEXTFIELD(cartridgeImage, ConfigureParams.Rom.szCartridgeImageFileName);
     EXPORT_RADIO(colorDepth, ConfigureParams.Screen.nVdiColors);
     EXPORT_SWITCH(compatibleCpu, ConfigureParams.System.bCompatibleCpu);
