@@ -33,6 +33,11 @@ const char IKBD_fileid[] = "Hatari ikbd.c : " __DATE__ " " __TIME__;
 /*			After a reset command, returns $F1 after $F0 (needed by Dragonnels Demo).	*/
 /*			This fixes the Transbeauce 2 demo menu, the Dragonnels demo menu and the	*/
 /*			Froggies Over The Fence demo menu (yeah ! enjoy this master piece of demo !).	*/
+/* 2011/05/11	[NP]	Add proper support for emulating TX buffer empty/full in status register bit 1	*/
+/*			when writing to $fffc02 (using an internal timer).				*/
+/* 2011/07/14	[NP]	Don't clear bytes in transit when ACIA_Reset is called ; if a byte is sent to	*/
+/*			the ikbd it should not be cancelled ? FIXME : this would need more tests on a	*/
+/*			real ST (fix Froggies Over The Fence's menu when selecting a demo).		*/
 
 
 #include <time.h>
@@ -45,6 +50,7 @@ const char IKBD_fileid[] = "Hatari ikbd.c : " __DATE__ " " __TIME__;
 #include "m68000.h"
 #include "memorySnapShot.h"
 #include "mfp.h"
+#include "screen.h"
 #include "video.h"
 #include "utils.h"
 
@@ -69,7 +75,7 @@ KEYBOARD Keyboard;
 KEYBOARD_PROCESSOR KeyboardProcessor;   /* Keyboard processor details */
 
 /* Pattern of mouse button up/down in ST frames (run off a double-click message) */
-static const uint8_t DoubleClickPattern[] =
+static const Uint8 DoubleClickPattern[] =
 {
 	BUTTON_MOUSE,BUTTON_MOUSE,BUTTON_MOUSE,BUTTON_MOUSE,
 	0,0,0,0,BUTTON_MOUSE,BUTTON_MOUSE,BUTTON_MOUSE,BUTTON_MOUSE
@@ -83,7 +89,9 @@ static bool bMouseEnabledDuringReset;
 static Uint8 ACIAControlRegister = 0;
 static Uint8 ACIAStatusRegister = ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY;  /* Pass when read 0xfffc00 */
 static Uint8 ACIAByte;				/* When a byte has arrived at the ACIA (from the keyboard) it is stored here */
+static Uint8 ACIATxDataRegister;		/* When a byte must be sent by the ACIA (to the keyboard) it is stored here */
 static bool bByteInTransitToACIA = false;	/* Is a byte being sent to the ACIA from the keyboard? */
+static bool bByteInTransitFromACIA = false;	/* Is a byte being sent from the ACIA to the keyboard? */
 
 /*
   6850 ACIA (Asynchronous Communications Inferface Apdater)
@@ -162,7 +170,7 @@ static bool bByteInTransitToACIA = false;	/* Is a byte being sent to the ACIA fr
   ST ACIA:-
     Note CTS,DCD and RTS are not connected! Phew!
     The keyboard ACIA are address 0xfffc000 and 0xfffc02.
-    Default parameters are :- 8-bit word, 1 stopbit, no parity, 77812.5 baud; 500KHz/64 (keyboard clock div)
+    Default parameters are :- 8-bit word, 1 stopbit, no parity, 7812.5 baud; 500KHz/64 (keyboard clock div)
     Default MIDI parameters are are above but :- 31250 baud; 500KHz/16 (MIDI clock div)
 */
 
@@ -202,7 +210,12 @@ static void IKBD_Cmd_ReportMouseAvailability(void);
 static void IKBD_Cmd_ReportJoystickMode(void);
 static void IKBD_Cmd_ReportJoystickAvailability(void);
 
-static const IKBD_COMMAND_PARAMS KeyboardCommands[] =
+/* Keyboard Command */
+static const struct {
+  Uint8 Command;
+  Uint8 NumParameters;
+  void (*pCallFunction)(void);
+} KeyboardCommands[] =
 {
 	/* Known messages, counts include command byte */
 	{ 0x80,2,  IKBD_Cmd_Reset },
@@ -272,21 +285,23 @@ static void IKBD_CustomCodeHandler_Transbeauce2Menu_Read ( void );
 static void IKBD_CustomCodeHandler_Transbeauce2Menu_Write ( Uint8 aciabyte );
 static void IKBD_CustomCodeHandler_DragonnelsMenu_Read ( void );
 static void IKBD_CustomCodeHandler_DragonnelsMenu_Write ( Uint8 aciabyte );
+static void IKBD_CustomCodeHandler_ChaosAD_Read ( void );
+static void IKBD_CustomCodeHandler_ChaosAD_Write ( Uint8 aciabyte );
 
 
-int	MemoryLoadNbBytesTotal = 0;			/* total number of bytes to send with the command 0x20 */
-int	MemoryLoadNbBytesLeft = 0;			/* number of bytes that remain to be sent  */
-Uint32	MemoryLoadCrc = 0xffffffff;			/* CRC of the bytes sent to the ikbd */
-int	MemoryExeNbBytes = 0;				/* current number of bytes sent to the ikbd when IKBD_ExeMode is true */
+static int	MemoryLoadNbBytesTotal = 0;		/* total number of bytes to send with the command 0x20 */
+static int	MemoryLoadNbBytesLeft = 0;		/* number of bytes that remain to be sent  */
+static Uint32	MemoryLoadCrc = 0xffffffff;		/* CRC of the bytes sent to the ikbd */
+static int	MemoryExeNbBytes = 0;			/* current number of bytes sent to the ikbd when IKBD_ExeMode is true */
 
-void	(*pIKBD_CustomCodeHandler_Read) ( void );
-void	(*pIKBD_CustomCodeHandler_Write) ( Uint8 );
-bool	IKBD_ExeMode = false;
+static void	(*pIKBD_CustomCodeHandler_Read) ( void );
+static void	(*pIKBD_CustomCodeHandler_Write) ( Uint8 );
+static bool	IKBD_ExeMode = false;
 
-Uint8	ScanCodeState[ 128 ];				/* state of each key : 0=released 1=pressed */
+static Uint8	ScanCodeState[ 128 ];			/* state of each key : 0=released 1=pressed */
 
 /* This array contains all known custom 6301 programs, with their CRC */
-struct
+static const struct
 {
 	Uint32		LoadMemCrc;			/* CRC of the bytes sent using the command 0x20 */
 	void		(*ExeBootHandler) ( Uint8 );	/* function handling write to $fffc02 during the 'boot' mode */
@@ -324,6 +339,15 @@ CustomCodeDefinitions[] =
 		IKBD_CustomCodeHandler_DragonnelsMenu_Read ,
 		IKBD_CustomCodeHandler_DragonnelsMenu_Write ,
 		"Dragonnels Main Menu"
+	},
+	{
+		0x9ad7fcdf ,
+		IKBD_CustomCodeHandler_CommonBoot ,
+		109 ,
+		0xa11d8be5 ,
+		IKBD_CustomCodeHandler_ChaosAD_Read ,
+		IKBD_CustomCodeHandler_ChaosAD_Write ,
+		"Chaos A.D."
 	}
 };
 
@@ -335,7 +359,14 @@ CustomCodeDefinitions[] =
  */
 void ACIA_Reset(void)
 {
+	LOG_TRACE(TRACE_IKBD_EXEC, "ikbd acia reset\n");
+
+/* [NP] 2011/07/14 FIXME : Acia reset should not clear bytes in transit ? */
+/* Else, "Froggies Over The Fence" doesn't exit the custom ikbd mode */
+#if 0
 	bByteInTransitToACIA = false;
+	bByteInTransitFromACIA = false;
+#endif
 	ACIAControlRegister = 0;
 	ACIAStatusRegister = ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY;
 }
@@ -362,6 +393,7 @@ void IKBD_Reset_ExeMode ( void )
 
 	Keyboard.BufferHead = Keyboard.BufferTail = 0;	/* flush all queued bytes that would be read in $fffc02 */
 	bByteInTransitToACIA = false;
+	bByteInTransitFromACIA = false;
 	// IKBD_AddKeyToKeyboardBuffer(0xF0);		/* Assume OK, return correct code */
 	IKBD_AddKeyToKeyboardBuffer(0xF1);		/* [NP] Dragonnels demo needs this */
 }
@@ -442,7 +474,9 @@ void IKBD_MemorySnapShot_Capture(bool bSave)
 	MemorySnapShot_Store(&ACIAControlRegister, sizeof(ACIAControlRegister));
 	MemorySnapShot_Store(&ACIAStatusRegister, sizeof(ACIAStatusRegister));
 	MemorySnapShot_Store(&ACIAByte, sizeof(ACIAByte));
+	MemorySnapShot_Store(&ACIATxDataRegister, sizeof(ACIATxDataRegister));
 	MemorySnapShot_Store(&bByteInTransitToACIA, sizeof(bByteInTransitToACIA));
+	MemorySnapShot_Store(&bByteInTransitFromACIA, sizeof(bByteInTransitFromACIA));
 	MemorySnapShot_Store(&bMouseDisabled, sizeof(bMouseDisabled));
 	MemorySnapShot_Store(&bJoystickDisabled, sizeof(bJoystickDisabled));
 	MemorySnapShot_Store(&bDuringResetCriticalTime, sizeof(bDuringResetCriticalTime));
@@ -942,12 +976,16 @@ static void IKBD_SendAutoKeyboardCommands(void)
  */
 void IKBD_InterruptHandler_AutoSend(void)
 {
-	/* Remove this interrupt from list and re-order */
-	CycInt_AcknowledgeInterrupt();
-
 	/* Handle user events and other messages, (like quit message) */
 	Main_EventHandler();
 
+	/* Remove this interrupt from list and re-order.
+	 * (needs to be done after UI event handling so
+	 * that snapshots saved from UI and restored from
+	 * command line don't miss the AUTOSEND interrupt)
+	 */
+	CycInt_AcknowledgeInterrupt();
+
 	/* Did user try to quit? */
 	if (bQuitProgram)
 	{
@@ -1398,11 +1436,23 @@ static void IKBD_Cmd_StopJoystick(void)
  */
 static void IKBD_Cmd_ReturnJoystick(void)
 {
+	/* The game "Downfall" continually issues this command during the
+	 * title screen - faster than processing the 3 bytes from the ACIA,
+	 * so it floods our Keyboard.Buffer ... in the end, we can not put the
+	 * whole packet into the buffer anymore, and the game hangs due to
+	 * these incomplete answers. To avoid this situation, only execute
+	 * this command if there is enough space left in our Keyboard.Buffer! */
+	if (((Keyboard.BufferHead-1-Keyboard.BufferTail)&KEYBOARD_BUFFER_MASK) < 3)
+	{
+		LOG_TRACE(TRACE_IKBD_CMDS, "IKBD_Cmd_ReturnJoystick ignored - buffer is full!\n");
+		return;
+	}
+
+	LOG_TRACE(TRACE_IKBD_CMDS, "IKBD_Cmd_ReturnJoystick\n");
+
 	IKBD_AddKeyToKeyboardBufferWithDelay(0xFD, 35000);
 	IKBD_AddKeyToKeyboardBuffer(Joy_GetStickData(0));
 	IKBD_AddKeyToKeyboardBuffer(Joy_GetStickData(1));
-
-	LOG_TRACE(TRACE_IKBD_CMDS, "IKBD_Cmd_ReturnJoystick\n");
 }
 
 
@@ -1504,11 +1554,14 @@ static void IKBD_Cmd_SetClock(void)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Convert value to 2-digit BCD
+ * Convert value to 2-digit BCD.
+ * Note: TOS 2.0x requires BCD conversion with overflow, so the decade
+ * is not calculated modulo 10, or it will end up in the year 2039
+ * instead...
  */
-static unsigned char IKBD_ConvertToBCD(unsigned short int Value)
+static Uint8 IKBD_ConvertToBCD(Uint16 Value)
 {
-	return (((Value/10))<<4) | (Value%10);
+	return ((Value/10)<<4) | (Value%10);
 }
 
 
@@ -1520,12 +1573,6 @@ static unsigned char IKBD_ConvertToBCD(unsigned short int Value)
  *   Returns:
  *     0xFC  ; time-of-day event header
  *     YY    ; year (2 least significant digits)
- *     There seems to be a problem with the bcd conversion of the year
- *     when year/10 >= 10. So the bcd conversion keeps the part > 10.
- *     If you put year%100 here (as says the doc), and put a real bcd
- *     conversion function in misc.c, then you end up with year 2031
- *     instead of 2003...
- *
  *     MM    ; month
  *     DD    ; day
  *     hh    ; hour
@@ -1604,7 +1651,8 @@ static void IKBD_Cmd_ReadMemory(void)
  */
 static void IKBD_Cmd_Execute(void)
 {
-	LOG_TRACE(TRACE_IKBD_CMDS, "IKBD_Cmd_Execute\n");
+	LOG_TRACE(TRACE_IKBD_CMDS, "IKBD_Cmd_Execute addr 0x%x\n",
+		(Keyboard.InputBuffer[1] << 8) + Keyboard.InputBuffer[2]);
 
 	if ( pIKBD_CustomCodeHandler_Write )
 	{
@@ -1911,37 +1959,54 @@ Uint16 IKBD_GetByteFromACIA(void)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Byte received in the ACIA from the keyboard processor. Store byte for read
- * from $fffc02 and schedule MFP interrupt to be triggered later.
+ * This interrupt handler is used to simulate a correct number of CPU cycles
+ * when sending a byte from the ACIA to the ikdb or when receiving a byte
+ * from the ikbd to the ACIA (because ACIA serial transfers are running
+ * at 7812.5 bps, bytes should not be available immediatly as it would break
+ * some programs)
+ * - Byte received in the ACIA from the keyboard processor : store byte for read
+ *   from $fffc02 and schedule MFP interrupt to be triggered just a few cycles after.
+ * - Byte sent from the ACIA to the keyboard processor : pass the byte to
+ *   IKBD_SendByteToKeyboardProcessor and set TX_BUFFER_EMPTY in status register.
  */
 void IKBD_InterruptHandler_ACIA(void)
 {
 	/* Remove this interrupt from list and re-order */
 	CycInt_AcknowledgeInterrupt();
 
-	/* Copy keyboard byte, ready for read from $fffc02 */
-	ACIAByte = Keyboard.Buffer[Keyboard.BufferHead++];
-	Keyboard.BufferHead &= KEYBOARD_BUFFER_MASK;
+	if ( bByteInTransitToACIA )
+	{
+		/* Copy keyboard byte, ready for read from $fffc02 */
+		ACIAByte = Keyboard.Buffer[Keyboard.BufferHead++];
+		Keyboard.BufferHead &= KEYBOARD_BUFFER_MASK;
 
-	/* Did we get an over-run? Ie byte has arrived from keyboard processor BEFORE CPU has read previous one from ACIA */
-	if (ACIAStatusRegister&ACIA_STATUS_REGISTER__RX_BUFFER_FULL)
-		ACIAStatusRegister |= ACIA_STATUS_REGISTER__OVERRUN_ERROR;  /* Set over-run */
+		/* Did we get an over-run? Ie byte has arrived from keyboard processor BEFORE CPU has read previous one from ACIA */
+		if (ACIAStatusRegister&ACIA_STATUS_REGISTER__RX_BUFFER_FULL)
+			ACIAStatusRegister |= ACIA_STATUS_REGISTER__OVERRUN_ERROR;  /* Set over-run */
 
-	/* ACIA buffer is now full */
-	ACIAStatusRegister |= ACIA_STATUS_REGISTER__RX_BUFFER_FULL;
-	/* Signal interrupt pending */
-	ACIAStatusRegister |= ACIA_STATUS_REGISTER__INTERRUPT_REQUEST;
+		/* ACIA buffer is now full */
+		ACIAStatusRegister |= ACIA_STATUS_REGISTER__RX_BUFFER_FULL;
+		/* Signal interrupt pending */
+		ACIAStatusRegister |= ACIA_STATUS_REGISTER__INTERRUPT_REQUEST;
 
-	/* GPIP I4 - General Purpose Pin Keyboard/MIDI interrupt */
-	/* NOTE: GPIP will remain low(0) until keyboard data is read from $fffc02. */
-	MFP_GPIP &= ~0x10;
+		/* GPIP I4 - General Purpose Pin Keyboard/MIDI interrupt */
+		/* NOTE: GPIP will remain low(0) until keyboard data is read from $fffc02. */
+		MFP_GPIP &= ~0x10;
 
-	/* There seems to be a small gap on a real ST between the point in time
-	 * the ACIA_STATUS_REGISTER__RX_BUFFER_FULL bit is set and the MFP
-	 * interrupt is triggered - for example the "V8 music system" demo
-	 * depends on this behaviour. To emulate this, we simply start another
-	 * Int which triggers the MFP interrupt later: */
-	CycInt_AddRelativeInterrupt(18, INT_CPU_CYCLE, INTERRUPT_IKBD_MFP);
+		/* There seems to be a small gap on a real ST between the point in time
+		* the ACIA_STATUS_REGISTER__RX_BUFFER_FULL bit is set and the MFP
+		* interrupt is triggered - for example the "V8 music system" demo
+		* depends on this behaviour. To emulate this, we simply start another
+		* Int which triggers the MFP interrupt later: */
+		CycInt_AddRelativeInterrupt(18, INT_CPU_CYCLE, INTERRUPT_IKBD_MFP);
+	}
+
+	if ( bByteInTransitFromACIA )
+	{
+		IKBD_SendByteToKeyboardProcessor(ACIATxDataRegister);  		/* Pass the byte to the keyboard processor */
+		ACIAStatusRegister |= ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY;	/* TX buffer is now empty */
+		bByteInTransitFromACIA = false;					/* ready to send another byte */
+	}
 }
 
 
@@ -2036,6 +2101,10 @@ static void IKBD_AddKeyToKeyboardBuffer_Real(Uint8 Data, int nAciaCycles)
 		/* We have character ready to transmit from the ACIA - see if can send it now */
 		IKBD_SendByteToACIA(nAciaCycles);
 	}
+	else
+	{
+		Log_Printf(LOG_ERROR, "IKBD buffer is full!\n");
+	}
 }
 
 
@@ -2066,10 +2135,6 @@ void IKBD_KeyboardControl_ReadByte(void)
 
 	IoMem[0xfffc00] = ACIAStatusRegister;
 
-	/* For our emulation send is immediate, so let's set the acknowledge
-	 * buffer empty bit for the next time the program polls the register */
-	ACIAStatusRegister |= ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY;
-
 	if (LOG_TRACE_LEVEL(TRACE_IKBD_ACIA))
 	{
 		int FrameCycles, HblCounterVideo, LineCycles;
@@ -2148,13 +2213,32 @@ void IKBD_KeyboardData_WriteByte(void)
 				IoMem[0xfffc02], FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
 	}
 
-	IKBD_SendByteToKeyboardProcessor(IoMem[0xfffc02]);  /* Pass our byte to the keyboard processor */
 
-	/* Some games like USS John Young / FOF54 actually check whether the
-	 * transmit-buffer-empty bit is really cleared after writing a data
-	 * byte to the IKBD, so we have to temporarily clear this bit, too,
-	 * although the byte is send immediately to our virtual IKBD. */
-	ACIAStatusRegister &= ~ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY;
+	if ( bByteInTransitFromACIA == false )			/* we're not already transfering a byte to the ikbd */
+	{
+//		IKBD_SendByteToKeyboardProcessor(IoMem[0xfffc02]);  /* Pass our byte to the keyboard processor */
+		ACIATxDataRegister = IoMem[0xfffc02];		/* store the byte that we want to send to the ikbd */
+
+		/* Delay the processing of the byte in IKBD_InterruptHandler_ACIA */
+		/* The delay doesn't seem to be constant, so we add a small random number of max 40 cycles */
+		/* (else some programs get stuck in an endless loop ('Pandemonium Demos' by Chaos) */
+
+		// [NP] FIXME 2011/05/20 : if we use a delay of ACIA_CYCLES=7200, tos 1.02 and 1.04 are showing
+		// a bug where addr $6122/$6124 are overwritten by the stack, preventing the desktop
+		// to be restored at the correct resolution !
+		// For now, use a delay of 1000 cycles ; need to do complete measures on a real ST for this
+		//CycInt_AddRelativeInterrupt(ACIA_CYCLES+rand()%40, INT_CPU_CYCLE, INTERRUPT_IKBD_ACIA);
+// delay cycles <=1560 ok, > 1570 bad for tos 1.02/1.04
+		CycInt_AddRelativeInterrupt(1000+rand()%40, INT_CPU_CYCLE, INTERRUPT_IKBD_ACIA);
+
+		/* Some games like USS John Young / FOF54 actually check whether the
+		* transmit-buffer-empty bit is really cleared after writing a data
+		* byte to the IKBD, so we have to temporarily clear this bit, too,
+		* although the byte is send immediately to our virtual IKBD. */
+		ACIAStatusRegister &= ~ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY;		/* TX buffer is full */
+
+		bByteInTransitFromACIA = true;
+	}
 }
 
 
@@ -2374,3 +2458,63 @@ static void IKBD_CustomCodeHandler_DragonnelsMenu_Write ( Uint8 aciabyte )
   /* Ignore write */
 }
 
+
+
+/*----------------------------------------------------------------------*/
+/* Chaos A.D. protection's decoder					*/
+/* This custom program reads bytes, decode them and send back the result*/
+/* to the 68000.							*/
+/* The program first returns $fe to indicate it's ready to receive the	*/
+/* encoded bytes.							*/
+/* The program then receives the 8 bytes used to decode the data and	*/
+/* store them in $f0 - $f7 (KeyBuffer is already initialized, so we	*/
+/* ignore those 8 bytes).						*/
+/* Then for any received byte a XOR is made with one of the byte in the	*/
+/* 8 bytes buffer, by incrementing an index in this buffer.		*/
+/* The decoded byte is written to addr $13 (TDR) to be received by ACIA	*/
+/*----------------------------------------------------------------------*/
+
+static void IKBD_CustomCodeHandler_ChaosAD_Read ( void )
+{
+	static bool	FirstCall = true;
+
+	if ( FirstCall == true )
+		IKBD_AddKeyToKeyboardBuffer_Real ( 0xfe , ACIA_CYCLES );
+
+	FirstCall = false;
+}
+
+static void IKBD_CustomCodeHandler_ChaosAD_Write ( Uint8 aciabyte )
+{
+	static int	IgnoreNb = 8;
+	Uint8		KeyBuffer[] = { 0xca , 0x0a , 0xbc , 0x00 , 0xde , 0xde , 0xfe , 0xca };
+	static int	Index = 0;
+	static int	Count = 0;
+
+	/* We ignore the first 8 bytes we received (they're already in KeyBuffer) */
+	if ( IgnoreNb > 0 )
+	{
+		IgnoreNb--;
+		return;
+	}
+
+	if ( Count <= 6080 )						/* there're 6081 bytes to decode */
+	{
+		Count++;
+		
+		aciabyte ^= KeyBuffer[ Index ];
+		Index++;
+		Index &= 0x07;
+
+		IKBD_AddKeyToKeyboardBuffer_Real ( aciabyte , ACIA_CYCLES );
+	}
+
+	else
+	{
+		/* When all bytes were decoded if 0x08 is written to $fffc02 */
+		/* the program will terminate itself and leave Execution mode */
+		if ( aciabyte == 0x08 )
+			IKBD_Reset_ExeMode ();
+	}
+}
+
