@@ -9,6 +9,7 @@
 const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 
 #include <time.h>
+#include <errno.h>
 #include <SDL.h>
 
 #include "main.h"
@@ -33,6 +34,7 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "paths.h"
 #include "printer.h"
 #include "reset.h"
+#include "resolution.h"
 #include "rs232.h"
 #include "screen.h"
 #include "sdlgui.h"
@@ -46,16 +48,21 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "video.h"
 #include "avi_record.h"
 #include "debugui.h"
+#include "clocks_timings.h"
 
 #include "hatari-glue.h"
 
 #include "falcon/hostscreen.h"
 #include "falcon/dsp.h"
 
+#if HAVE_GETTIMEOFDAY
+#include <sys/time.h>
+#endif
+
 
 bool bQuitProgram = false;                /* Flag to quit program cleanly */
 
-Uint32 nRunVBLs;                          /* Whether and how many VBLS to run before exit */
+static Uint32 nRunVBLs;                   /* Whether and how many VBLS to run before exit */
 static Uint32 nFirstMilliTick;            /* Ticks when VBL counting started */
 static Uint32 nVBLCount;                  /* Frame count */
 
@@ -96,6 +103,58 @@ static Uint32 Main_GetTicks(void)
 #endif
 
 
+//#undef HAVE_GETTIMEOFDAY
+//#undef HAVE_NANOSLEEP
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Return a time counter in micro seconds.
+ * If gettimeofday is available, we use it directly, else we convert the
+ * return of SDL_GetTicks in micro sec.
+ */
+
+static Sint64	Time_GetTicks ( void )
+{
+        Sint64		ticks_micro;
+
+#if HAVE_GETTIMEOFDAY
+        struct timeval	now;
+        gettimeofday ( &now , NULL );
+        ticks_micro = (Sint64)now.tv_sec * 1000000 + now.tv_usec;
+#else
+	ticks_micro = (Sint64)SDL_GetTicks() * 1000;		/* milli sec -> micro sec */
+#endif
+
+	return ticks_micro;
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Sleep for a given number of micro seconds.
+ * If nanosleep is available, we use it directly, else we use SDL_Delay
+ * (which is portable, but less accurate as is uses milli-seconds)
+ */
+
+static void	Time_Delay ( Sint64 ticks_micro )
+{
+#if HAVE_NANOSLEEP
+	struct timespec	ts;
+	int		ret;
+	ts.tv_sec = ticks_micro / 1000000;
+	ts.tv_nsec = (ticks_micro % 1000000) * 1000;	/* micro sec -> nano sec */
+	/* wait until all the delay is elapsed, including possible interruptions by signals */
+	do
+	{
+                errno = 0;
+                ret = nanosleep(&ts, &ts);
+	} while ( ret && ( errno == EINTR ) );		/* keep on sleeping if we were interrupted */
+#else
+	SDL_Delay ( (Uint32)(ticks_micro / 1000) ) ;	/* micro sec -> milli sec */
+#endif
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Pause emulation, stop sound.  'visualize' should be set true,
@@ -121,7 +180,7 @@ bool Main_PauseEmulation(bool visualize)
 			current = (1000.0 * nVBLCount) / interval;
 			printf("SPEED: %.1f VBL/s (%d/%.1fs), diff=%.1f%%\n",
 			       current, nVBLCount, interval/1000.0,
-			       previous ? 100*(current-previous)/previous : 0.0);
+			       previous>0.0 ? 100*(current-previous)/previous : 0.0);
 			nVBLCount = nFirstMilliTick = 0;
 			previous = current;
 		}
@@ -148,7 +207,7 @@ bool Main_UnPauseEmulation(void)
 	if ( bEmulationActive )
 		return false;
 
-	Sound_ResetBufferIndex();
+	Sound_BufferIndexNeedReset = true;
 	Audio_EnableAudio(ConfigureParams.Sound.bEnableSound);
 	bEmulationActive = true;
 
@@ -189,6 +248,18 @@ void Main_RequestQuit(void)
 	}
 }
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Set how many VBLs Hatari should run, from the moment this function
+ * is called.
+ */
+void Main_SetRunVBLs(Uint32 vbls)
+{
+	fprintf(stderr, "Exit after %d VBLs.\n", vbls);
+	nRunVBLs = vbls;
+	nVBLCount = 0;
+}
+
 /*-----------------------------------------------------------------------*/
 /**
  * This function waits on each emulated VBL to synchronize the real time
@@ -197,13 +268,14 @@ void Main_RequestQuit(void)
  * are very inaccurate on some systems like Linux 2.4 or Mac OS X (they can only
  * wait for a multiple of 10ms due to the scheduler on these systems), so we have
  * to "busy wait" there to get an accurate timing.
+ * All times are expressed as micro seconds, to avoid too much rounding error.
  */
 void Main_WaitOnVbl(void)
 {
-	int nCurrentMilliTicks;
-	static int nDestMilliTicks = 0;
-	int nFrameDuration;
-	signed int nDelay;
+	Sint64 CurrentTicks;
+	static Sint64 DestTicks = 0;
+	Sint64 FrameDuration_micro;
+	Sint64 nDelay;
 
 	nVBLCount++;
 	if (nRunVBLs &&	nVBLCount >= nRunVBLs)
@@ -212,14 +284,19 @@ void Main_WaitOnVbl(void)
 		Main_PauseEmulation(true);
 		exit(0);
 	}
-	nCurrentMilliTicks = SDL_GetTicks();
 
-	nFrameDuration = 1000/nScreenRefreshRate;
-	nDelay = nDestMilliTicks - nCurrentMilliTicks;
+//	FrameDuration_micro = (Sint64) ( 1000000.0 / nScreenRefreshRate + 0.5 );	/* round to closest integer */
+	FrameDuration_micro = ClocksTimings_GetVBLDuration_micro ( ConfigureParams.System.nMachineType , nScreenRefreshRate );
+	CurrentTicks = Time_GetTicks();
+
+	if ( DestTicks == 0 )					/* first call, init DestTicks */
+		DestTicks = CurrentTicks + FrameDuration_micro;
+
+	nDelay = DestTicks - CurrentTicks;
 
 	/* Do not wait if we are in fast forward mode or if we are totally out of sync */
 	if (ConfigureParams.System.bFastForward == true
-	        || nDelay < -4*nFrameDuration)
+	        || nDelay < -4*FrameDuration_micro)
 	{
 		if (ConfigureParams.System.bFastForward == true)
 		{
@@ -231,8 +308,8 @@ void Main_WaitOnVbl(void)
 			nFrameSkips += 1;
 			// Log_Printf(LOG_DEBUG, "Increased frameskip to %d\n", nFrameSkips);
 		}
-		/* Only update nDestMilliTicks for next VBL */
-		nDestMilliTicks = nCurrentMilliTicks + nFrameDuration;
+		/* Only update DestTicks for next VBL */
+		DestTicks = CurrentTicks + FrameDuration_micro;
 		return;
 	}
 	/* If automatic frameskip is enabled and delay's more than twice
@@ -240,7 +317,7 @@ void Main_WaitOnVbl(void)
 	 */
 	if (nFrameSkips > 0
 	    && ConfigureParams.Screen.nFrameSkips >= AUTO_FRAMESKIP_LIMIT
-	    && 2*nDelay > nFrameDuration/nFrameSkips)
+	    && 2*nDelay > FrameDuration_micro/nFrameSkips)
 	{
 		nFrameSkips -= 1;
 		// Log_Printf(LOG_DEBUG, "Decreased frameskip to %d\n", nFrameSkips);
@@ -249,25 +326,26 @@ void Main_WaitOnVbl(void)
 	if (bAccurateDelays)
 	{
 		/* Accurate sleeping is possible -> use SDL_Delay to free the CPU */
-		if (nDelay > 1)
-			SDL_Delay(nDelay - 1);
+		if (nDelay > 1000)
+			Time_Delay(nDelay - 1000);
 	}
 	else
 	{
 		/* No accurate SDL_Delay -> only wait if more than 5ms to go... */
-		if (nDelay > 5)
-			SDL_Delay(nDelay<10 ? nDelay-1 : 9);
+		if (nDelay > 5000)
+			Time_Delay(nDelay<10000 ? nDelay-1000 : 9000);
 	}
 
 	/* Now busy-wait for the right tick: */
 	while (nDelay > 0)
 	{
-		nCurrentMilliTicks = SDL_GetTicks();
-		nDelay = nDestMilliTicks - nCurrentMilliTicks;
+		CurrentTicks = Time_GetTicks();
+		nDelay = DestTicks - CurrentTicks;
 	}
 
-	/* Update nDestMilliTicks for next VBL */
-	nDestMilliTicks += nFrameDuration;
+//printf ( "tick %lld\n" , CurrentTicks );
+	/* Update DestTicks for next VBL */
+	DestTicks += FrameDuration_micro;
 }
 
 
@@ -468,6 +546,18 @@ void Main_EventHandler(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Set Hatari window title. Use NULL for default
+ */
+void Main_SetTitle(const char *title)
+{
+	if (title)
+		SDL_WM_SetCaption(title, "Hatari");
+	else
+		SDL_WM_SetCaption(PROG_NAME, "Hatari");
+}
+
 /*-----------------------------------------------------------------------*/
 /**
  * Initialise emulation
@@ -489,12 +579,14 @@ static void Main_Init(void)
 		fprintf(stderr, "Could not initialize the SDL library:\n %s\n", SDL_GetError() );
 		exit(-1);
 	}
-
+	ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
+	Resolution_Init();
 	SDLGui_Init();
 	Printer_Init();
 	RS232_Init();
 	Midi_Init();
 	Screen_Init();
+	Main_SetTitle(NULL);
 	HostScreen_Init();
 	DSP_Init();
 	Floppy_Init();
@@ -647,7 +739,7 @@ int main(int argc, char *argv[])
 	Main_LoadInitialConfig();
 
 	/* Check for any passed parameters */
-	if (!Opt_ParseParameters(argc, (const char**)argv))
+	if (!Opt_ParseParameters(argc, (const char * const *)argv))
 	{
 		return 1;
 	}
@@ -675,7 +767,12 @@ int main(int argc, char *argv[])
 	Main_CheckForAccurateDelays();
 
 	if ( AviRecordOnStartup )	/* Immediatly starts avi recording ? */
-		Avi_StartRecording ( AviRecordFile , AviRecordDefaultCrop , AviRecordDefaultFps , AviRecordDefaultVcodec );
+		Avi_StartRecording ( ConfigureParams.Video.AviRecordFile , ConfigureParams.Screen.bCrop ,
+			ConfigureParams.Video.AviRecordFps == 0 ?
+				ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , nScreenRefreshRate ) :
+				(Uint32)ConfigureParams.Video.AviRecordFps << CLOCKS_TIMINGS_SHIFT_VBL ,
+			1 << CLOCKS_TIMINGS_SHIFT_VBL ,
+			ConfigureParams.Video.AviRecordVcodec );
 
 	/* Run emulation */
 	Main_UnPauseEmulation();
