@@ -44,18 +44,22 @@ const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c : " __DATE__ " " _
 #include "str.h"
 #include "stMemory.h"
 #include "tos.h"
+#include "screen.h"
 #include "video.h"
 #include "falcon/dsp.h"
+#include "falcon/crossbar.h"
+#include "falcon/videl.h"
+#include "statusbar.h"
 
 
-#define VERSION_STRING      "1.4.0"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
-#define VERSION_STRING_SIZE    6      /* Size of above (inc' NULL) */
-
+#define VERSION_STRING      "1.5.0"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
 
 #define COMPRESS_MEMORYSNAPSHOT       /* Compress snapshots to reduce disk space used */
 
 #ifdef COMPRESS_MEMORYSNAPSHOT
 
+/* Remove possible conflicting mkdir declaration from cpu/sysdeps.h */
+#undef mkdir
 #include <zlib.h>
 typedef gzFile MSS_File;
 
@@ -133,14 +137,19 @@ static int MemorySnapShot_fwrite(MSS_File fhndl, const char *buf, int len)
  */
 static bool MemorySnapShot_OpenFile(const char *pszFileName, bool bSave)
 {
-	char VersionString[VERSION_STRING_SIZE];
+	char VersionString[] = VERSION_STRING;
 
 	/* Set error */
 	bCaptureError = false;
 
-	/* Open file, set flag so 'MemorySnapShot_Store' can load to/save from file */
+	/* after opening file, set bCaptureSave to indicate whether
+	 * 'MemorySnapShot_Store' should load from or save to a file
+	 */
 	if (bSave)
 	{
+		if (!File_QueryOverwrite(pszFileName))
+			return false;
+
 		/* Save */
 		CaptureFile = MemorySnapShot_fopen(pszFileName, "wb");
 		if (!CaptureFile)
@@ -152,8 +161,7 @@ static bool MemorySnapShot_OpenFile(const char *pszFileName, bool bSave)
 		}
 		bCaptureSave = true;
 		/* Store version string */
-		strcpy(VersionString, VERSION_STRING);
-		MemorySnapShot_Store(VersionString, VERSION_STRING_SIZE);
+		MemorySnapShot_Store(VersionString, sizeof(VersionString));
 	}
 	else
 	{
@@ -168,13 +176,14 @@ static bool MemorySnapShot_OpenFile(const char *pszFileName, bool bSave)
 		}
 		bCaptureSave = false;
 		/* Restore version string */
-		MemorySnapShot_Store(VersionString, VERSION_STRING_SIZE);
+		MemorySnapShot_Store(VersionString, sizeof(VersionString));
 		/* Does match current version? */
 		if (strcasecmp(VersionString, VERSION_STRING))
 		{
 			/* No, inform user and error */
-			Log_AlertDlg(LOG_ERROR, "Unable to Restore Memory State.\nFile is "
-			                       "only compatible with Hatari v%s", VersionString);
+			Log_AlertDlg(LOG_ERROR, "Unable to restore Hatari memory state. File\n"
+			                       "is compatible only with Hatari version %s.",
+				     VersionString);
 			bCaptureError = true;
 			return false;
 		}
@@ -245,11 +254,17 @@ void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
 		Video_MemorySnapShot_Capture(true);
 		Blitter_MemorySnapShot_Capture(true);
 		DmaSnd_MemorySnapShot_Capture(true);
+		Crossbar_MemorySnapShot_Capture(true);
+		VIDEL_MemorySnapShot_Capture(true);
 		DSP_MemorySnapShot_Capture(true);
 		DebugUI_MemorySnapShot_Capture(pszFileName, true);
-
+		IoMem_MemorySnapShot_Capture(true);
 		/* And close */
 		MemorySnapShot_CloseFile();
+	} else {
+		/* just canceled? */
+		if (!bCaptureError)
+			return;
 	}
 
 	/* Did error */
@@ -291,11 +306,17 @@ void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
 		Video_MemorySnapShot_Capture(false);
 		Blitter_MemorySnapShot_Capture(false);
 		DmaSnd_MemorySnapShot_Capture(false);
+		Crossbar_MemorySnapShot_Capture(false);
+		VIDEL_MemorySnapShot_Capture(false);
 		DSP_MemorySnapShot_Capture(false);
 		DebugUI_MemorySnapShot_Capture(pszFileName, false);
+		IoMem_MemorySnapShot_Capture(false);
 
 		/* And close */
 		MemorySnapShot_CloseFile();
+
+		/* changes may affect also info shown in statusbar */
+		Statusbar_UpdateInfo();
 	}
 
 	/* Did error? */
