@@ -118,6 +118,7 @@ const char PSG_fileid[] = "Hatari psg.c : " __DATE__ " " __TIME__;
 #if ENABLE_DSP_EMU
 #include "falcon/dsp.h"
 #endif
+#include "screen.h"
 #include "video.h"
 #include "statusbar.h"
 #include "mfp.h"
@@ -239,7 +240,7 @@ void PSG_Set_DataRegister(Uint8 val)
 		return;					/* not valid, ignore write and do nothing */
 
 	/* Create samples up until this point with current values */
-	Sound_Update();
+	Sound_Update(false);
 
 	/* Copy value to PSGRegisters[] */
 	PSGRegisters[PSGRegisterSelect] = val;
