@@ -542,7 +542,6 @@ void Screen_Init(void)
 	Video_SetScreenRasters();                       /* Set rasters ready for first screen */
 
 	/* Configure some SDL stuff: */
-	SDL_WM_SetCaption(PROG_NAME, "Hatari");
 	SDL_ShowCursor(SDL_DISABLE);
 }
 
@@ -1124,13 +1123,17 @@ static bool Screen_DrawFrame(bool bForceFlip)
 				bPrevFrameWasSpec512 = true;
 				/* What mode were we in? Keep to 320xH or 640xH */
 				if (pDrawFunction==ConvertLowRes_320x16Bit)
-					pDrawFunction = ConvertSpec512_320x16Bit;
+					pDrawFunction = ConvertLowRes_320x16Bit_Spec;
 				else if (pDrawFunction==ConvertLowRes_640x16Bit)
-					pDrawFunction = ConvertSpec512_640x16Bit;
+					pDrawFunction = ConvertLowRes_640x16Bit_Spec;
 				else if (pDrawFunction==ConvertLowRes_320x32Bit)
-					pDrawFunction = ConvertSpec512_320x32Bit;
+					pDrawFunction = ConvertLowRes_320x32Bit_Spec;
 				else if (pDrawFunction==ConvertLowRes_640x32Bit)
-					pDrawFunction = ConvertSpec512_640x32Bit;
+					pDrawFunction = ConvertLowRes_640x32Bit_Spec;
+				else if (pDrawFunction==ConvertMediumRes_640x32Bit)
+					pDrawFunction = ConvertMediumRes_640x32Bit_Spec;
+				else if (pDrawFunction==ConvertMediumRes_640x16Bit)
+					pDrawFunction = ConvertMediumRes_640x16Bit_Spec;
 			}
 			else if (bPrevFrameWasSpec512)
 			{
@@ -1250,23 +1253,25 @@ static void Convert_StartFrame(void)
 
 /* Conversion routines */
 
-#include "convert/low320x8.c"     /* LowRes To 320xH x 8-bit color */
-#include "convert/low640x8.c"     /* LowRes To 640xH x 8-bit color */
-#include "convert/med640x8.c"     /* MediumRes To 640xH x 8-bit color */
-#include "convert/high640x8.c"    /* HighRes To 640xH x 8-bit color */
+#include "convert/low320x8.c"		/* LowRes To 320xH x 8-bit color */
+#include "convert/low640x8.c"		/* LowRes To 640xH x 8-bit color */
+#include "convert/med640x8.c"		/* MediumRes To 640xH x 8-bit color */
+#include "convert/high640x8.c"		/* HighRes To 640xH x 8-bit color */
 
-#include "convert/low320x16.c"    /* LowRes To 320xH x 16-bit color */
-#include "convert/low640x16.c"    /* LowRes To 640xH x 16-bit color */
-#include "convert/med640x16.c"    /* MediumRes To 640xH x 16-bit color */
-#include "convert/spec320x16.c"   /* Spectrum 512 To 320xH x 16-bit color */
-#include "convert/spec640x16.c"   /* Spectrum 512 To 640xH x 16-bit color */
+#include "convert/low320x16.c"		/* LowRes To 320xH x 16-bit color */
+#include "convert/low640x16.c"		/* LowRes To 640xH x 16-bit color */
+#include "convert/med640x16.c"		/* MediumRes To 640xH x 16-bit color */
+#include "convert/low320x16_spec.c"	/* LowRes Spectrum 512 To 320xH x 16-bit color */
+#include "convert/low640x16_spec.c"	/* LowRes Spectrum 512 To 640xH x 16-bit color */
+#include "convert/med640x16_spec.c"	/* MediumRes Spectrum 512 To 640xH x 16-bit color */
 
-#include "convert/low320x32.c"    /* LowRes To 320xH x 32-bit color */
-#include "convert/low640x32.c"    /* LowRes To 640xH x 32-bit color */
-#include "convert/med640x32.c"    /* MediumRes To 640xH x 32-bit color */
-#include "convert/spec320x32.c"   /* Spectrum 512 To 320xH x 32-bit color */
-#include "convert/spec640x32.c"   /* Spectrum 512 To 640xH x 32-bit color */
+#include "convert/low320x32.c"		/* LowRes To 320xH x 32-bit color */
+#include "convert/low640x32.c"		/* LowRes To 640xH x 32-bit color */
+#include "convert/med640x32.c"		/* MediumRes To 640xH x 32-bit color */
+#include "convert/low320x32_spec.c"	/* LowRes Spectrum 512 To 320xH x 32-bit color */
+#include "convert/low640x32_spec.c"	/* LowRes Spectrum 512 To 640xH x 32-bit color */
+#include "convert/med640x32_spec.c"	/* MediumRes Spectrum 512 To 640xH x 32-bit color */
 
-#include "convert/vdi16.c"        /* VDI x 16 color */
-#include "convert/vdi4.c"         /* VDI x 4 color */
-#include "convert/vdi2.c"         /* VDI x 2 color */
+#include "convert/vdi16.c"		/* VDI x 16 color */
+#include "convert/vdi4.c"		/* VDI x 4 color */
+#include "convert/vdi2.c"		/* VDI x 2 color */
