@@ -28,6 +28,7 @@ const char ShortCut_fileid[] = "Hatari shortcut.c : " __DATE__ " " __TIME__;
 #include "sdlgui.h"
 #include "video.h"
 #include "avi_record.h"
+#include "clocks_timings.h"
 
 static SHORTCUTKEYIDX ShortCutKey = SHORTCUT_NONE;  /* current shortcut key */
 
@@ -111,7 +112,12 @@ static void ShortCut_RecordAnimation(void)
 	else
 	{
 		/* Start animation */
-		Avi_StartRecording ( AviRecordFile , AviRecordDefaultCrop , nScreenRefreshRate , AviRecordDefaultVcodec );
+		Avi_StartRecording ( ConfigureParams.Video.AviRecordFile , ConfigureParams.Screen.bCrop ,
+			ConfigureParams.Video.AviRecordFps == 0 ?
+				ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , nScreenRefreshRate ) :
+				(Uint32)ConfigureParams.Video.AviRecordFps << CLOCKS_TIMINGS_SHIFT_VBL ,
+			1 << CLOCKS_TIMINGS_SHIFT_VBL ,
+			ConfigureParams.Video.AviRecordVcodec );
 	}
 }
 
@@ -152,7 +158,7 @@ static void ShortCut_FastForward(void)
 		ConfigureParams.System.bFastForward = false;
 
 		/* Reset the sound emulation variables: */
-		Sound_ResetBufferIndex();
+		Sound_BufferIndexNeedReset = true;
 	}
 	else
 	{
