@@ -23,6 +23,10 @@
     - 5 bits volume table and 16*16*16 combinations of all volume are from Sc68 by Benjamin Gerard
     - 4 bits to 5 bits volume interpolation from 16*16*16 to 32*32*32 from YM blep synthesis by Antti Lankila
 
+  Special case for per==0 : as measured on a real STF, when tone/noise/env's per==0, we get
+  the same sound output as when per==1.
+
+
 */
 
 /* 2008/05/05	[NP]	Fix case where period is 0 for noise, sound or envelope.	*/
@@ -81,10 +85,12 @@ const char Sound_fileid[] = "Hatari sound.c : " __DATE__ " " __TIME__;
 #include "memorySnapShot.h"
 #include "psg.h"
 #include "sound.h"
+#include "screen.h"
 #include "video.h"
 #include "wavFormat.h"
 #include "ymFormat.h"
 #include "avi_record.h"
+#include "clocks_timings.h"
 
 
 
@@ -161,13 +167,13 @@ static yms16 *ymout5 = (yms16 *)ymout5_u16;
 /*--------------------------------------------------------------*/
 
 /* Number of generated samples per frame (eg. 44Khz=882) */
-#define SAMPLES_PER_FRAME  ((nAudioFrequency+35)/nScreenRefreshRate)
+#define SAMPLES_PER_FRAME  (nAudioFrequency/nScreenRefreshRate)
 
 /* Current sound replay freq (usually 44100 Hz) */
 #define YM_REPLAY_FREQ   nAudioFrequency
 
-/* YM-2149 clock on Atari ST is 2 MHz */
-#define YM_ATARI_CLOCK		(CPU_FREQ / 4)
+/* YM-2149 clock on all Atari models is 2 MHz */
+#define YM_ATARI_CLOCK		(MachineClocks.YM_Freq)
 
 
 /* Merge/read the 3 volumes in a single integer (5 bits per volume) */
@@ -223,16 +229,21 @@ static ymu16	Vol3Voices = 0;				/* volume 0-0x1f for voices having a constant vo
 /* Global variables that can be changed/read from other parts of Hatari */
 Uint8		SoundRegs[ 14 ];
 
-int		YmVolumeMixing = YM_LINEAR_MIXING;
+int		YmVolumeMixing = YM_TABLE_MIXING;
 bool		UseLowPassFilter = false;
 
 bool		bEnvelopeFreqFlag;			/* Cleared each frame for YM saving */
 
 Sint16		MixBuffer[MIXBUFFER_SIZE][2];
 int		nGeneratedSamples;			/* Generated samples since audio buffer update */
-int		nSamplesToGenerate;			/* How many samples are needed for this time-frame */
 static int	ActiveSndBufIdx;			/* Current working index into above mix buffer */
+static int	ActiveSndBufIdxAvi;			/* Current working index to save an AVI audio frame */
 
+static yms64	SamplesPerFrame_unrounded = 0;		/* Number of samples for the current VBL, with simulated fractional part */
+static int 	SamplesPerFrame;			/* Number of samples to generate for the current VBL */
+static int	CurrentSamplesNb = 0;			/* Number of samples already generated for the current VBL */
+
+bool		Sound_BufferIndexNeedReset = false;
 
 
 /*--------------------------------------------------------------*/
@@ -254,6 +265,7 @@ static void	YM2149_BuildLinearVolumeTable(ymu16 *out);
 static void	YM2149_Normalise_5bit_Table(ymu16 *in_5bit , yms16 *out_5bit, unsigned int Level, bool DoCenter);
 
 static void	YM2149_EnvBuild		(void);
+static void	Ym2149_BuildVolumeTable	(void);
 static void	Ym2149_Init		(void);
 static void	Ym2149_Reset		(void);
 
@@ -263,8 +275,8 @@ static ymu32	Ym2149_NoiseStepCompute	(ymu8 rNoise);
 static ymu32	Ym2149_EnvStepCompute	(ymu8 rHigh , ymu8 rLow);
 static ymsample	YM2149_NextSample	(void);
 
-static void	Sound_SetSamplesPassed	(void);
-static void	Sound_GenerateSamples	(void);
+static int	Sound_SetSamplesPassed(bool FillFrame);
+static void	Sound_GenerateSamples(int SamplesToGenerate);
 
 
 
@@ -535,6 +547,27 @@ static void	YM2149_EnvBuild ( void )
 
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Depending on the YM mixing method, build the table used to convert
+ * the 3 YM volumes into a single sample.
+ */
+
+static void	Ym2149_BuildVolumeTable(void)
+{
+	/* Depending on the volume mixing method, we use a table based on real measures */
+	/* or a table based on a linear volume mixing. */
+	if ( YmVolumeMixing == YM_TABLE_MIXING )
+		interpolate_volumetable(ymout5_u16);		/* expand the 16*16*16 values in volumetable_original to 32*32*32 */
+	else
+		YM2149_BuildLinearVolumeTable(ymout5_u16);	/* combine the 32 possible volumes */
+
+	/* Normalise/center the values (convert from u16 to s16) */
+	YM2149_Normalise_5bit_Table ( ymout5_u16 , ymout5 , YM_OUTPUT_LEVEL , YM_OUTPUT_CENTERED );
+}
+
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Init some internal tables for faster results (env, volume)
@@ -546,15 +579,8 @@ static void	Ym2149_Init(void)
 	/* Build the 16 envelope shapes */
 	YM2149_EnvBuild();
 
-	/* Depending on the volume mixing method, we use a table based on real measures */
-	/* or a table based on a linear volume mixing. */
-	if ( YmVolumeMixing == YM_TABLE_MIXING )
-		interpolate_volumetable(ymout5_u16);	/* expand the 16*16*16 values in volumetable_original to 32*32*32 */
-	else
-		YM2149_BuildLinearVolumeTable(ymout5_u16);	/* combine the 32 possible volumes */
-
-	/* Normalise/center the values (convert from u16 to s16) */
-	YM2149_Normalise_5bit_Table ( ymout5_u16 , ymout5 , YM_OUTPUT_LEVEL , YM_OUTPUT_CENTERED );
+	/* Build the volume conversion table */
+	Ym2149_BuildVolumeTable();
 
 	/* Reset YM2149 internal states */
 	Ym2149_Reset();
@@ -564,7 +590,7 @@ static void	Ym2149_Init(void)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Reset all ym registers as well as the internal varaibles
+ * Reset all ym registers as well as the internal variables
  */
 
 static void	Ym2149_Reset(void)
@@ -576,6 +602,10 @@ static void	Ym2149_Reset(void)
 
 	Sound_WriteReg ( 7 , 0xff );
 
+	posA = 0;
+	posB = 0;
+	posC = 0;
+
 	currentNoise = 0xffff;
 
 	RndRack = 1;
@@ -607,10 +637,19 @@ static ymu32	YM2149_RndCompute(void)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Compute steps for tone, noise and env, based on the input
- * period.
+ * Compute tone's step based on the input period.
+ * Although for tone we should have the same result when per==0 and per==1,
+ * this gives some very sharp and unpleasant sounds in the emulation.
+ * To get a better sound, we consider all per<=5 to give step=0, which will
+ * produce a constant output at value '1'. This should be handled with some
+ * proper filters to remove high frequencies as on a real ST (where per<=9
+ * gives nearly no audible sound).
+ * A common replay freq of 44.1 kHz will also not be high enough to correctly
+ * render possible tone's freq of 125 or 62.5 kHz (when per==1 or per==2)
  */
 
+#define NEWSTEP
+#ifndef NEWSTEP
 static ymu32	Ym2149_ToneStepCompute(ymu8 rHigh , ymu8 rLow)
 {
 	int	per;
@@ -627,8 +666,46 @@ static ymu32	Ym2149_ToneStepCompute(ymu8 rHigh , ymu8 rLow)
 
 	return step;
 }
+#else
+static ymu32	Ym2149_ToneStepCompute(ymu8 rHigh , ymu8 rLow)
+{
+	int	per;
+	yms64	step;
 
+	per = rHigh&15;
+	per = (per<<8)+rLow;
 
+#if 0							/* need some high freq filters for this to work correctly */
+	if ( per == 0 )
+		per = 1;				/* result for Per=0 is the same as for Per=1 */	
+#else
+	if ( per <= 5 )
+		return 0;				/* discard too high frequencies, they give a very bad sound */	
+#endif
+
+	step = YM_ATARI_CLOCK;
+	step <<= 24;
+
+	step /= (per * 8 * YM_REPLAY_FREQ);		/* 0x5ab9 < step < 0x5ab3f46 at 44.1 kHz */
+
+	return step;
+}
+#endif
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Compute noise's step based on the input period.
+ * On a real STF, we get the same result when per==0 and per==1.
+ * A common replay freq of 44.1 kHz will not be high enough to correctly
+ * render possible noise's freq of 125 or 62.5 kHz (when per==1 or per==2).
+ * With a random wave such as noise, this means that with a replay freq
+ * of 44.1 kHz, per==1 and per==2 (as well as per==3) will sound the same :
+ * 	per==1   step=0x2d59fa3   freq=125 kHz
+ * 	per==2   step=0x16acfd1   freq=62.5 kHz
+ * 	per==3   step=0x0f1dfe1   freq=41.7 kHz
+ */
+
+#ifndef NEWSTEP
 static ymu32	Ym2149_NoiseStepCompute(ymu8 rNoise)
 {
 	int	per;
@@ -644,7 +721,25 @@ static ymu32	Ym2149_NoiseStepCompute(ymu8 rNoise)
 
 	return step;
 }
+#else
+static ymu32	Ym2149_NoiseStepCompute(ymu8 rNoise)
+{
+	int	per;
+	yms64	step;
 
+	per = (rNoise&0x1f);
+
+	if ( per == 0 )
+		per = 1;				/* result for Per=0 is the same as for Per=1 */	
+
+	step = YM_ATARI_CLOCK;
+	step <<= 24;
+
+	step /= (per * 16 * YM_REPLAY_FREQ);		/* 0x17683f < step < 0x2d59fa3 at 44.1 kHz */
+
+	return step;
+}
+#endif
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -661,6 +756,7 @@ static ymu32	Ym2149_NoiseStepCompute(ymu8 rNoise)
  * the equivalent of 24 bits for the fractional part.
  * Since we're using large numbers, we temporarily use 64 bits integer
  * to avoid overflow and keep largest precision possible.
+ * On a real STF, we get the same result when per==0 and per==1.
  */
 
 static ymu32	Ym2149_EnvStepCompute(ymu8 rHigh , ymu8 rLow)
@@ -673,10 +769,11 @@ static ymu32	Ym2149_EnvStepCompute(ymu8 rHigh , ymu8 rLow)
 
 	step = YM_ATARI_CLOCK;
 	step <<= 24;
-	if ( per > 0 )
-		step /= (8 * per * YM_REPLAY_FREQ);	/* 0x5ab < step < 0x5ab3f46 at 44.1 kHz */
-	else
-		step /= (8 * 1/2 * YM_REPLAY_FREQ);	/* result for Per=0 is half the result for Per=1 */
+
+	if ( per == 0 )
+		per = 1;				/* result for Per=0 is the same as for Per=1 */	
+
+	step /= (8 * per * YM_REPLAY_FREQ);		/* 0x5ab < step < 0x5ab3f46 at 44.1 kHz */
 
 	return step;
 }
@@ -688,8 +785,16 @@ static ymu32	Ym2149_EnvStepCompute(ymu8 rHigh , ymu8 rLow)
  * Main function : compute the value of the next sample.
  * Mixes all 3 voices with tone+noise+env and apply low pass
  * filter if needed.
+ * All operations are done with integer math, using <<24 to simulate
+ * floating point precision : upper 8 bits are the integer part, lower 24
+ * are the fractional part.
+ * Tone is a square wave with 2 states 0 or 1. If integer part of posX is
+ * even (bit24=0) we consider output is 0, else (bit24=1) we consider
+ * output is 1. This gives the value of bt for one voice after extending it
+ * to all 0 bits or all 1 bits using a '-'
  */
 
+#ifndef NEWSTEP
 static ymsample	YM2149_NextSample(void)
 {
 	ymsample	sample;
@@ -750,15 +855,83 @@ static ymsample	YM2149_NextSample(void)
 
 	return sample;
 }
+#else
+static ymsample	YM2149_NextSample(void)
+{
+	ymsample	sample;
+	ymu32		bt;
+	ymu32		bn;
+	ymu16		Env3Voices;			/* 0x00CCBBAA */
+	ymu16		Tone3Voices;			/* 0x00CCBBAA */
 
 
+	/* Noise value : 0 or 0xffff */
+	if ( noisePos&0xff000000 )			/* integer part > 0 */
+	{
+		currentNoise ^= YM2149_RndCompute();
+		noisePos &= 0xffffff;			/* keep fractional part of noisePos */
+	}
+	bn = currentNoise;				/* 0 or 0xffff */
+
+	/* Get the 5 bits volume corresponding to the current envelope's position */
+	Env3Voices = YmEnvWaves[ envShape ][ envPos>>24 ];	/* integer part of envPos is in bits 24-31 */
+	Env3Voices &= EnvMask3Voices;			/* only keep volumes for voices using envelope */
+
+//fprintf ( stderr , "env %x %x %x\n" , Env3Voices , envStep , envPos );
+
+	/* Tone3Voices will contain the output state of each voice : 0 or 0x1f */
+	bt = -( (posA>>24) & 1);			/* 0 if bit24=0 or 0xffffffff if bit24=1 */
+	bt = (bt | mixerTA) & (bn | mixerNA);		/* 0 or 0xffff */
+	Tone3Voices = bt & YM_MASK_1VOICE;		/* 0 or 0x1f */
+	bt = -( (posB>>24) & 1);
+	bt = (bt | mixerTB) & (bn | mixerNB);
+	Tone3Voices |= ( bt & YM_MASK_1VOICE ) << 5;
+	bt = -( (posC>>24) & 1);
+	bt = (bt | mixerTC) & (bn | mixerNC);
+	Tone3Voices |= ( bt & YM_MASK_1VOICE ) << 10;
+
+	/* Combine fixed volumes and envelope volumes and keep the resulting */
+	/* volumes depending on the output state of each voice (0 or 0x1f) */
+	Tone3Voices &= ( Env3Voices | Vol3Voices );
+
+	/* D/A conversion of the 3 volumes into a sample using a precomputed conversion table */
+	sample = ymout5[ Tone3Voices ];			/* 16 bits signed value */
+
+
+	/* Increment positions */
+	posA += stepA;
+	posB += stepB;
+	posC += stepC;
+	noisePos += noiseStep;
+
+	envPos += envStep;
+	if ( envPos >= (3*32) << 24 )			/* blocks 0, 1 and 2 were used (envPos 0 to 95) */
+		envPos -= (2*32) << 24;			/* replay/loop blocks 1 and 2 (envPos 32 to 95) */
+
+	DcAdjuster_AddSample(sample);			/* Calculate DC level */
+	sample = sample - DcAdjuster_GetDcLevel();	/* normalize sound level */
+
+	/* Apply low pass filter ? */
+	if ( UseLowPassFilter )
+	{
+		sample = LowPassFilter(sample);
+	}
+
+	return sample;
+}
+#endif
+
 
 /*-----------------------------------------------------------------------*/
 /**
  * Update internal variables (steps, volume masks, ...) each
  * time an YM register is changed.
  */
-
+#ifndef NEWSTEP
+#define BIT_SHIFT 31
+#else
+#define BIT_SHIFT 24
+#endif
 void	Sound_WriteReg( int reg , Uint8 data )
 {
 	switch (reg)
@@ -766,37 +939,37 @@ void	Sound_WriteReg( int reg , Uint8 data )
 		case 0:
 			SoundRegs[0] = data;
 			stepA = Ym2149_ToneStepCompute ( SoundRegs[1] , SoundRegs[0] );
-			if (!stepA) posA = 1u<<31;		// Assume output always 1 if 0 period (for Digi-sample)
+			if (!stepA) posA = 1u<<BIT_SHIFT;		// Assume output always 1 if 0 period (for Digi-sample)
 			break;
 
 		case 1:
 			SoundRegs[1] = data & 0x0f;
 			stepA = Ym2149_ToneStepCompute ( SoundRegs[1] , SoundRegs[0] );
-			if (!stepA) posA = 1u<<31;		// Assume output always 1 if 0 period (for Digi-sample)
+			if (!stepA) posA = 1u<<BIT_SHIFT;		// Assume output always 1 if 0 period (for Digi-sample)
 			break;
 
 		case 2:
 			SoundRegs[2] = data;
 			stepB = Ym2149_ToneStepCompute ( SoundRegs[3] , SoundRegs[2] );
-			if (!stepB) posB = 1u<<31;		// Assume output always 1 if 0 period (for Digi-sample)
+			if (!stepB) posB = 1u<<BIT_SHIFT;		// Assume output always 1 if 0 period (for Digi-sample)
 			break;
 
 		case 3:
 			SoundRegs[3] = data & 0x0f;
 			stepB = Ym2149_ToneStepCompute ( SoundRegs[3] , SoundRegs[2] );
-			if (!stepB) posB = 1u<<31;		// Assume output always 1 if 0 period (for Digi-sample)
+			if (!stepB) posB = 1u<<BIT_SHIFT;		// Assume output always 1 if 0 period (for Digi-sample)
 			break;
 
 		case 4:
 			SoundRegs[4] = data;
 			stepC = Ym2149_ToneStepCompute ( SoundRegs[5] , SoundRegs[4] );
-			if (!stepC) posC = 1u<<31;		// Assume output always 1 if 0 period (for Digi-sample)
+			if (!stepC) posC = 1u<<BIT_SHIFT;		// Assume output always 1 if 0 period (for Digi-sample)
 			break;
 
 		case 5:
 			SoundRegs[5] = data & 0x0f;
 			stepC = Ym2149_ToneStepCompute ( SoundRegs[5] , SoundRegs[4] );
-			if (!stepC) posC = 1u<<31;		// Assume output always 1 if 0 period (for Digi-sample)
+			if (!stepC) posC = 1u<<BIT_SHIFT;		// Assume output always 1 if 0 period (for Digi-sample)
 			break;
 
 		case 6:
@@ -921,6 +1094,9 @@ void Sound_Reset(void)
 	/* We do not start with 0 here to fake some initial samples: */
 	nGeneratedSamples = SoundBufferSize + SAMPLES_PER_FRAME;
 	ActiveSndBufIdx = nGeneratedSamples % MIXBUFFER_SIZE;
+	SamplesPerFrame = SAMPLES_PER_FRAME;
+	CurrentSamplesNb = 0;
+	ActiveSndBufIdxAvi = ActiveSndBufIdx;
 //fprintf ( stderr , "Sound_Reset SoundBufferSize %d SAMPLES_PER_FRAME %d nGeneratedSamples %d , ActiveSndBufIdx %d\n" ,
 //	SoundBufferSize , SAMPLES_PER_FRAME, nGeneratedSamples , ActiveSndBufIdx );
 
@@ -933,12 +1109,20 @@ void Sound_Reset(void)
 /*-----------------------------------------------------------------------*/
 /**
  * Reset the sound buffer index variables.
+ * Very important : this function should only be called by setting
+ * Sound_BufferIndexNeedReset=true ; sound buffer index should be reset
+ * only after the sound for the whole VBL was updated (CurrentSamplesNb returns to 0)
+ * else it will alter the value of DMA Frame Count ($ff8909/0b/0d) and
+ * could cause crashes in some programs.
  */
 void Sound_ResetBufferIndex(void)
 {
 	Audio_Lock();
 	nGeneratedSamples = SoundBufferSize + SAMPLES_PER_FRAME;
 	ActiveSndBufIdx =  (CompleteSndBufIdx + nGeneratedSamples) % MIXBUFFER_SIZE;
+	SamplesPerFrame = SAMPLES_PER_FRAME;
+	CurrentSamplesNb = 0;
+	ActiveSndBufIdxAvi = ActiveSndBufIdx;
 //fprintf ( stderr , "Sound_ResetBufferIndex SoundBufferSize %d SAMPLES_PER_FRAME %d nGeneratedSamples %d , ActiveSndBufIdx %d\n" ,
 //	SoundBufferSize , SAMPLES_PER_FRAME, nGeneratedSamples , ActiveSndBufIdx );
 	Audio_Unlock();
@@ -990,33 +1174,62 @@ void Sound_MemorySnapShot_Capture(bool bSave)
  * Find how many samples to generate and store in 'nSamplesToGenerate'
  * Also update sound cycles counter to store how many we actually did
  * so generates set amount each frame.
+ * If FillFrame is true, this means we reach the end of the VBL and me must
+ * add as many samples as necessary to get a total of SamplesPerFrame
+ * for this VBL.
  */
-static void Sound_SetSamplesPassed(void)
+static int Sound_SetSamplesPassed(bool FillFrame)
 {
-	int nSampleCycles;
-	int nSamplesPerFrame;
 	int nSoundCycles;
+	int SamplesToGenerate;				/* How many samples are needed for this time-frame */
 
-	nSoundCycles = Cycles_GetCounter(CYCLES_COUNTER_SOUND);
+	nSoundCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
 
-	/* 160256 cycles per VBL, 44Khz = 882 samples per VBL */
-	/* 882/160256 samples per clock cycle */
-	nSamplesPerFrame = SAMPLES_PER_FRAME;
+	/* example : 160256 cycles per VBL, 44Khz = 882 samples per VBL at 50 Hz */
+	/* 882/160256 samples per cpu clock cycle */
 
-	nSamplesToGenerate = nSoundCycles * nSamplesPerFrame / CYCLES_PER_FRAME;
-	if (nSamplesToGenerate > nSamplesPerFrame)
-		nSamplesToGenerate = nSamplesPerFrame;
+	/* Total number of samples that we should have at this point of the VBL */
+	SamplesToGenerate = nSoundCycles * SamplesPerFrame
+		/ ClocksTimings_GetCyclesPerVBL ( ConfigureParams.System.nMachineType , nScreenRefreshRate );
 
-	nSampleCycles = nSamplesToGenerate * CYCLES_PER_FRAME / nSamplesPerFrame;
-	nSoundCycles -= nSampleCycles;
-	Cycles_SetCounter(CYCLES_COUNTER_SOUND, nSoundCycles);
+//if (SamplesToGenerate > SamplesPerFrame )
+//fprintf ( stderr , "over run %d %d\n" , SamplesPerFrame , SamplesToGenerate );
 
-	if (nSamplesToGenerate > MIXBUFFER_SIZE - nGeneratedSamples)
+	if (SamplesToGenerate > SamplesPerFrame)
+		SamplesToGenerate = SamplesPerFrame;
+
+	SamplesToGenerate -= CurrentSamplesNb;		/* don't count samples that were already generated up to now */
+	if ( SamplesToGenerate < 0 )
+		SamplesToGenerate = 0;
+
+
+	/* If we're called from the VBL interrupt (FillFrame==true), we must ensure we have */
+	/* an exact total of SamplesPerFrame samples during a full VBL (we take into account */
+	/* the samples that were already generated during this VBL) */
+	if ( FillFrame )
 	{
-		nSamplesToGenerate = MIXBUFFER_SIZE - nGeneratedSamples;
-		if (nSamplesToGenerate < 0)
-			nSamplesToGenerate = 0;
+		SamplesToGenerate = SamplesPerFrame - CurrentSamplesNb;	/* how many samples are missing to reach SamplesPerFrame */
+		if ( SamplesToGenerate < 0 )
+			SamplesToGenerate = 0;
 	}
+
+	/* Check we don't fill the sound's ring buffer before it's played by Audio_Callback()	*/
+	/* This should never happen, except if the system suffers major slowdown due to	other	*/
+	/* processes or if we run in fast forward mode.						*/
+	/* In the case of slowdown, we set Sound_BufferIndexNeedReset to "resync" the working	*/
+	/* buffer's index ActiveSndBufIdx with the system buffer's index CompleteSndBufIdx.	*/
+	/* In the case of fast forward, we do nothing here, Sound_BufferIndexNeedReset will be	*/
+	/* set when the user exits fast forward mode.						*/
+	if ( ( SamplesToGenerate > MIXBUFFER_SIZE - nGeneratedSamples ) && ( ConfigureParams.System.bFastForward == false )
+	    && ( ConfigureParams.Sound.bEnableSound == true ) )
+	{
+		Log_Printf ( LOG_WARN , "Your system is too slow, some sound samples were not correctly emulated\n" );
+		Sound_BufferIndexNeedReset = true;
+	}
+
+//fprintf ( stderr , "vbl %d hbl %d samp_gen %d / %d frac %lx\n" , nVBLs , nHBL , SamplesToGenerate , SamplesPerFrame , (long int)SamplesPerFrame_unrounded );
+
+	return SamplesToGenerate;
 }
 
 
@@ -1024,37 +1237,37 @@ static void Sound_SetSamplesPassed(void)
 /**
  * Generate samples for all channels during this time-frame
  */
-static void Sound_GenerateSamples(void)
+static void Sound_GenerateSamples(int SamplesToGenerate)
 {
 	int	i;
 	int	idx;
 
-	if (nSamplesToGenerate <= 0)
+	if (SamplesToGenerate <= 0)
 		return;
 
 	if (ConfigureParams.System.nMachineType == MACHINE_FALCON)
 	{
-		for (i = 0; i < nSamplesToGenerate; i++)
+		for (i = 0; i < SamplesToGenerate; i++)
 		{
 			idx = (ActiveSndBufIdx + i) % MIXBUFFER_SIZE;
 			MixBuffer[idx][0] = MixBuffer[idx][1] = YM2149_NextSample();
 		}
  		/* If Falcon emulation, crossbar does the job */
- 		Crossbar_GenerateSamples(ActiveSndBufIdx, nSamplesToGenerate);
+ 		Crossbar_GenerateSamples(ActiveSndBufIdx, SamplesToGenerate);
 	}
 	else if (ConfigureParams.System.nMachineType != MACHINE_ST)
 	{
-		for (i = 0; i < nSamplesToGenerate; i++)
+		for (i = 0; i < SamplesToGenerate; i++)
 		{
 			idx = (ActiveSndBufIdx + i) % MIXBUFFER_SIZE;
 			MixBuffer[idx][0] = MixBuffer[idx][1] = (YM2149_NextSample() >> 1);
 		}
  		/* If Ste or TT emulation, DmaSnd does the job */
- 		DmaSnd_GenerateSamples(ActiveSndBufIdx, nSamplesToGenerate);
+ 		DmaSnd_GenerateSamples(ActiveSndBufIdx, SamplesToGenerate);
 	}
 	else if (ConfigureParams.System.nMachineType == MACHINE_ST)
 	{
-		for (i = 0; i < nSamplesToGenerate; i++)
+		for (i = 0; i < SamplesToGenerate; i++)
 		{
 			idx = (ActiveSndBufIdx + i) % MIXBUFFER_SIZE;
 			MixBuffer[idx][0] = MixBuffer[idx][1] = YM2149_NextSample();
@@ -1063,57 +1276,85 @@ static void Sound_GenerateSamples(void)
 		DmaSnd_GenerateSamples(ActiveSndBufIdx, nSamplesToGenerate); */
  	}
 
-	ActiveSndBufIdx = (ActiveSndBufIdx + nSamplesToGenerate) % MIXBUFFER_SIZE;
-	nGeneratedSamples += nSamplesToGenerate;
+	ActiveSndBufIdx = (ActiveSndBufIdx + SamplesToGenerate) % MIXBUFFER_SIZE;
+	nGeneratedSamples += SamplesToGenerate;
+	CurrentSamplesNb += SamplesToGenerate;				/* number of samples generated for current VBL */
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
  * This is called to built samples up until this clock cycle
+ * Sound_Update can be called several times during a VBL ; we must ensure
+ * that we generate exactly SamplesPerFrame samples between 2 calls
+ * to Sound_Update_VBL.
  */
-void Sound_Update(void)
+void Sound_Update(bool FillFrame)
 {
 	int OldSndBufIdx = ActiveSndBufIdx;
+	int SamplesToGenerate;
 
 	/* Make sure that we don't interfere with the audio callback function */
 	Audio_Lock();
 
-	/* Find how many to generate */
-	Sound_SetSamplesPassed();
+	/* Find how many samples to generate */
+	SamplesToGenerate = Sound_SetSamplesPassed( FillFrame );
+
 	/* And generate */
-	Sound_GenerateSamples();
+	Sound_GenerateSamples( SamplesToGenerate );
 
 	/* Allow audio callback function to occur again */
 	Audio_Unlock();
 
 	/* Save to WAV file, if open */
 	if (bRecordingWav)
-		WAVFormat_Update(MixBuffer, OldSndBufIdx, nSamplesToGenerate);
+		WAVFormat_Update(MixBuffer, OldSndBufIdx, SamplesToGenerate);
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * On each VBL (50fps) complete samples.
+ * On the end of each VBL, complete audio buffer to reach SamplesPerFrame samples.
+ * As Sound_Update(false) could be called several times during the VBL, the audio
+ * buffer might be already partially filled.
+ * We must first complete the buffer using the same value of SamplesPerFrame
+ * by calling Sound_Update(true) ; then we can compute a new value for
+ * SamplesPerFrame that will be used for the next VBL to come.
  */
 void Sound_Update_VBL(void)
 {
-	Sound_Update();
+	Sound_Update(true);					/* generate as many samples as needed to fill this VBL */
+//fprintf ( stderr , "vbl done %d %d\n" , SamplesPerFrame , CurrentSamplesNb );
 
-	/* Record audio frame is necessary */
+	CurrentSamplesNb = 0;					/* VBL is complete, reset counter for next VBL */
+
+	/*Compute a fractional equivalent of SamplesPerFrame for the next VBL, to avoid rounding propagation */
+	SamplesPerFrame_unrounded += (yms64) ClocksTimings_GetSamplesPerVBL ( ConfigureParams.System.nMachineType ,
+			nScreenRefreshRate , nAudioFrequency );
+	SamplesPerFrame = SamplesPerFrame_unrounded >> 28;		/* use integer part */
+	SamplesPerFrame_unrounded &= 0x0fffffff;			/* keep fractional part in the lower 28 bits */
+
+	/* Reset sound buffer if needed (after pause, fast forward, slow system, ...) */
+	if ( Sound_BufferIndexNeedReset )
+	{
+		Sound_ResetBufferIndex ();
+		Sound_BufferIndexNeedReset = false;
+	}
+	
+	/* Record AVI audio frame is necessary */
 	if ( bRecordingAvi )
 	{
-		int SamplePerVbl = nAudioFrequency / nScreenRefreshRate;
-		int PrevIndex;
+		int Len;
 
-		PrevIndex = ActiveSndBufIdx - SamplePerVbl;
-		if ( PrevIndex < 0 )
-			PrevIndex += MIXBUFFER_SIZE;
+		Len = ActiveSndBufIdx - ActiveSndBufIdxAvi;	/* number of generated samples for this frame */
+		if ( Len < 0 )
+			Len += MIXBUFFER_SIZE;			/* end of ring buffer was reached */
 
-		Avi_RecordAudioStream ( MixBuffer , PrevIndex , SamplePerVbl );
+		Avi_RecordAudioStream ( MixBuffer , ActiveSndBufIdxAvi , Len );
 	}
 
+	ActiveSndBufIdxAvi = ActiveSndBufIdx;			/* save new position for next AVI audio frame */
+
 	/* Clear write to register '13', used for YM file saving */
 	bEnvelopeFreqFlag = false;
 }
@@ -1172,3 +1413,14 @@ bool Sound_AreWeRecording(void)
 	return (bRecordingYM || bRecordingWav);
 }
 
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Rebuild volume conversion table
+ */
+void Sound_SetYmVolumeMixing(void)
+{
+	/* Build the volume conversion table */
+	Ym2149_BuildVolumeTable();
+}
+
