@@ -39,6 +39,7 @@ const char Statusbar_fileid[] = "Hatari statusbar.c : " __DATE__ " " __TIME__;
 #include "sdlgui.h"
 #include "statusbar.h"
 #include "tos.h"
+#include "screen.h"
 #include "video.h"
 #include "wavFormat.h"
 #include "ymFormat.h"
@@ -414,6 +415,9 @@ void Statusbar_UpdateInfo(void)
 	case MACHINE_STE:
 		end = Statusbar_AddString(end, "STE");
 		break;
+	case MACHINE_MEGA_STE:
+		end = Statusbar_AddString(end, "Mega STE");
+		break;
 	case MACHINE_TT:
 		end = Statusbar_AddString(end, "TT");
 		break;
@@ -439,6 +443,7 @@ void Statusbar_UpdateInfo(void)
 
 	assert(end - DefaultMessage.msg < MAX_MESSAGE_LEN);
 	DEBUGPRINT(("Set default message: '%s'\n", DefaultMessage.msg));
+	/* make sure default message gets (re-)drawn when next checked */
 	DefaultMessage.shown = false;
 }
 
@@ -470,7 +475,7 @@ static void Statusbar_ShowMessage(SDL_Surface *surf, Uint32 ticks)
 
 	if (MessageList->shown) {
 		if (!MessageList->expire) {
-			/* last/default message */
+			/* last/default message newer expires */
 			return;
 		}
 		if (MessageList->expire > ticks) {
