@@ -6,15 +6,16 @@
 
   Load TOS image file into ST memory, fix/setup for emulator.
 
-  The Atari ST TOS needs to be patched to help with emulation. Eg, it references
-  the MMU chip to set memory size. This is patched to the sizes we need without
-  the complicated emulation of hardware which is not needed (as yet). We also
-  patch DMA devices and Hard Drives.
-  NOTE: TOS versions 1.06 and 1.62 were not designed for use on a real STfm.
-  These were for the STe machine ONLY. They access the DMA/Microwire addresses
-  on boot-up which (correctly) cause a bus-error on Hatari as they would in a
-  real STfm. If a user tries to select any of these images we bring up an error.
-*/
+  The Atari ST TOS needs to be patched to help with emulation. Eg, it
+  references the MMU chip to set memory size. This is patched to the
+  sizes we need without the complicated emulation of hardware which
+  is not needed (as yet). We also patch DMA devices and Hard Drives.
+
+  NOTE: TOS versions 1.06 and 1.62 were not designed for use on a
+  real STfm. These were for the STe machine ONLY. They access the
+  DMA/Microwire addresses on boot-up which (correctly) cause a
+  bus-error on Hatari as they would in a real STfm. If a user tries
+  to select any of these images we bring up an error. */
 const char TOS_fileid[] = "Hatari tos.c : " __DATE__ " " __TIME__;
 
 #include <SDL_endian.h>
@@ -29,9 +30,11 @@ const char TOS_fileid[] = "Hatari tos.c : " __DATE__ " " __TIME__;
 #include "m68000.h"
 #include "memorySnapShot.h"
 #include "stMemory.h"
+#include "str.h"
 #include "tos.h"
 #include "vdi.h"
-
+#include "falcon/dsp.h"
+#include "clocks_timings.h"
 
 bool bIsEmuTOS;
 Uint16 TosVersion;                      /* eg. 0x0100, 0x0102 */
@@ -41,7 +44,6 @@ bool bRamTosImage;                      /* true if we loaded a RAM TOS image */
 unsigned int ConnectedDriveMask = 0x03; /* Bit mask of connected drives, eg 0x7 is A,B,C */
 int nNumDrives = 2;                     /* Number of drives, default is 2 for A: and B: - Strictly, this is the highest mapped drive letter, in-between drives may not be allocated */
 
-
 /* Possible TOS file extensions to scan for */
 static const char * const pszTosNameExts[] =
 {
@@ -51,6 +53,55 @@ static const char * const pszTosNameExts[] =
 	NULL
 };
 
+static struct {
+	FILE *file;          /* file pointer to contents of INF file */
+	char prgname[16];    /* TOS name of the program to auto start */
+	const char *infname; /* name of the INF file TOS will try to match */
+	int match_count;     /* how many times INF was matched after boot */
+	int match_max;       /* how many times TOS needs it to be matched */
+} TosAutoStart;
+
+/* autostarted program name will be added after first '\' character */
+static const char emudesk_inf[] =
+"#E 9A 07\r\n"
+"#Z 01 C:\\@\r\n"
+"#W 00 00 02 06 26 0C 08 C:\\*.*@\r\n"
+"#W 00 00 02 08 26 0C 00 @\r\n"
+"#W 00 00 02 0A 26 0C 00 @\r\n"
+"#W 00 00 02 0D 26 0C 00 @\r\n"
+"#M 00 00 01 FF A DISK A@ @\r\n"
+"#M 01 00 01 FF B DISK B@ @\r\n"
+"#M 02 00 01 FF C DISK C@ @\r\n"
+"#F FF 28 @ *.*@\r\n"
+"#D FF 02 @ *.*@\r\n"
+"#G 08 FF *.APP@ @\r\n"
+"#G 08 FF *.PRG@ @\r\n"
+"#P 08 FF *.TTP@ @\r\n"
+"#F 08 FF *.TOS@ @\r\n"
+"#T 00 03 03 FF   TRASH@ @\r\n";
+
+static const char desktop_inf[] =
+"#a000000\r\n"
+"#b001000\r\n"
+"#c7770007000600070055200505552220770557075055507703111302\r\n"
+"#d\r\n"
+"#Z 01 C:\\@\r\n"
+"#E D8 11\r\n"
+"#W 00 00 10 01 17 17 13 C:\\*.*@\r\n"
+"#W 00 00 08 0B 1D 0D 00 @\r\n"
+"#W 00 00 0A 0F 1A 09 00 @\r\n"
+"#W 00 00 0E 01 1A 09 00 @\r\n"
+"#M 00 00 05 FF A DISK A@ @\r\n"
+"#M 00 01 05 FF B DISK B@ @\r\n"
+"#M 00 02 05 FF C DISK C@ @\r\n"
+"#T 00 03 02 FF   TRASH@ @\r\n"
+"#F FF 04   @ *.*@\r\n"
+"#D FF 01   @ *.*@\r\n"
+"#P 03 04   @ *.*@\r\n"
+"#G 03 FF   *.APP@ @\r\n"
+"#G 03 FF   *.PRG@ @\r\n"
+"#P 03 FF   *.TTP@ @\r\n"
+"#F 03 04   *.TOS@ @\r\n";
 
 /* Flags that define if a TOS patch should be applied */
 enum
@@ -247,6 +298,130 @@ static void TOS_FixRom(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Set name of program that will be auto started after TOS boots.
+ * Supported only from TOS 1.04 forward.
+ */
+void TOS_AutoStart(const char *prgname)
+{
+	Str_Filename2TOSname(prgname, TosAutoStart.prgname);
+}
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Create a temporary desktop.inf file which will start autostart program.
+ * This needs to be re-created on each boot in case user changed TOS version.
+ */
+static void TOS_CreateAutoInf(void)
+{
+	const char *contents, *infname, *prgname;
+	int offset, size, max;
+	FILE *fp;
+
+	/* in case TOS didn't for some reason close it on previous boot */
+	TOS_AutoStartClose(TosAutoStart.file);
+
+	prgname = TosAutoStart.prgname;
+	/* autostart not enabled? */
+	if (!*prgname)
+		return;
+
+	/* autostart not supported? */
+	if (TosVersion < 0x0104)
+	{
+		Log_Printf(LOG_WARN, "Only TOS versions >= 1.04 support autostarting!\n");
+		return;
+	}
+
+	if (bIsEmuTOS)
+	{
+		infname = "C:\\EMUDESK.INF";
+		size = sizeof(emudesk_inf);
+		contents = emudesk_inf;
+		max = 1;
+	} else {
+		/* need to match file TOS searches first */
+		if (TosVersion >= 0x0200)
+			infname = "NEWDESK.INF";
+		else
+			infname = "DESKTOP.INF";
+		size = sizeof(desktop_inf);
+		contents = desktop_inf;
+		max = 1;
+	}
+	/* infname needs to be exactly the same string that given
+	 * TOS version gives for GEMDOS to find.
+	 */
+	TosAutoStart.infname = infname;
+	TosAutoStart.match_max = max;
+	TosAutoStart.match_count = 0;
+
+	/* find where to insert the program name */
+	for (offset = 0; offset < size; )
+	{
+		if (contents[offset++] == '\\')
+			break;
+	}
+	assert(offset < size);
+
+	/* create the autostart file */
+	fp = tmpfile();
+	if (!(fp
+	      && fwrite(contents, offset, 1, fp) == 1
+	      && fwrite(prgname, strlen(prgname), 1, fp) == 1
+	      && fwrite(contents+offset, size-offset-1, 1, fp) == 1
+	      && fseek(fp, 0, SEEK_SET) == 0))
+	{
+		if (fp)
+			fclose(fp);
+		Log_Printf(LOG_ERROR, "Failed to create autostart file for '%s'!\n", TosAutoStart.prgname);
+		return;
+	}
+	TosAutoStart.file = fp;
+	Log_Printf(LOG_WARN, "Virtual autostart file '%s' created for '%s'.\n", infname, prgname);
+}
+
+/*-----------------------------------------------------------------------*/
+/**
+ * If given name matches autostart file, return its handle, NULL otherwise
+ */
+FILE *TOS_AutoStartOpen(const char *filename)
+{
+	if (TosAutoStart.file && strcmp(filename, TosAutoStart.infname) == 0)
+	{
+		Log_Printf(LOG_WARN, "Autostart file '%s' for '%s' matched.\n", filename, TosAutoStart.prgname);
+		return TosAutoStart.file;
+	}
+	return NULL;
+}
+
+/*-----------------------------------------------------------------------*/
+/**
+ * If given handle matches autostart file, close it and return true,
+ * false otherwise.
+ */
+bool TOS_AutoStartClose(FILE *fp)
+{
+	if (fp && fp == TosAutoStart.file)
+	{
+		if (++TosAutoStart.match_count >= TosAutoStart.match_max)
+		{
+			/* Remove autostart INF file after TOS has
+			 * read it enough times to do autostarting.
+			 * Otherwise user may try change desktop settings
+			 * and save them, but they would be lost.
+			 */
+			fclose(TosAutoStart.file);
+			TosAutoStart.file = NULL;
+			Log_Printf(LOG_WARN, "Autostart file removed.\n");
+		}
+		return true;
+	}
+	return false;
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Assert that TOS version matches the machine type and change the system
@@ -265,10 +440,11 @@ static void TOS_CheckSysConfig(void)
 		             " ==> Switching to STE mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_STE;
-		ConfigureParams.System.nCpuFreq = 8;
+		ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
 		IoMem_Init();
+		ConfigureParams.System.nCpuFreq = 8;
 		ConfigureParams.System.nCpuLevel = 0;
-		M68000_CheckCpuLevel();
+		M68000_CheckCpuSettings();
 	}
 	else if ((TosVersion & 0x0f00) == 0x0300 && ConfigureParams.System.nMachineType != MACHINE_TT)
 	{
@@ -276,10 +452,11 @@ static void TOS_CheckSysConfig(void)
 		             " ==> Switching to TT mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_TT;
-		ConfigureParams.System.nCpuFreq = 32;
+		ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
 		IoMem_Init();
+		ConfigureParams.System.nCpuFreq = 32;
 		ConfigureParams.System.nCpuLevel = 3;
-		M68000_CheckCpuLevel();
+		M68000_CheckCpuSettings();
 	}
 	else if ((TosVersion & 0x0f00) == 0x0400 && ConfigureParams.System.nMachineType != MACHINE_FALCON)
 	{
@@ -287,10 +464,15 @@ static void TOS_CheckSysConfig(void)
 		             " ==> Switching to Falcon mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_FALCON;
-		ConfigureParams.System.nCpuFreq = 16;
+		ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
+#if ENABLE_DSP_EMU
+		ConfigureParams.System.nDSPType = DSP_TYPE_EMU;
+		DSP_Init();
+#endif
 		IoMem_Init();
+		ConfigureParams.System.nCpuFreq = 16;
 		ConfigureParams.System.nCpuLevel = 3;
-		M68000_CheckCpuLevel();
+		M68000_CheckCpuSettings();
 	}
 	else if (TosVersion <= 0x0104 && ConfigureParams.System.nCpuLevel > 0)
 	{
@@ -298,10 +480,11 @@ static void TOS_CheckSysConfig(void)
 		             " ==> Switching to ST mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_ST;
-		ConfigureParams.System.nCpuFreq = 8;
+		ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
 		IoMem_Init();
+		ConfigureParams.System.nCpuFreq = 8;
 		ConfigureParams.System.nCpuLevel = 0;
-		M68000_CheckCpuLevel();
+		M68000_CheckCpuSettings();
 	}
 	else if ((TosVersion < 0x0300 && ConfigureParams.System.nMachineType == MACHINE_FALCON)
 	         || (TosVersion < 0x0200 && ConfigureParams.System.nMachineType == MACHINE_TT))
@@ -310,17 +493,18 @@ static void TOS_CheckSysConfig(void)
 		             " ==> Switching to STE mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_STE;
-		ConfigureParams.System.nCpuFreq = 8;
+		ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
 		IoMem_Init();
+		ConfigureParams.System.nCpuFreq = 8;
 		ConfigureParams.System.nCpuLevel = 0;
-		M68000_CheckCpuLevel();
+		M68000_CheckCpuSettings();
 	}
 	else if (TosVersion >= 0x0300 && ConfigureParams.System.nCpuLevel < 2)
 	{
 		Log_AlertDlg(LOG_ERROR, "This TOS versions requires a CPU >= 68020.\n"
 		             " ==> Switching to 68020 mode now.\n");
 		ConfigureParams.System.nCpuLevel = 2;
-		M68000_CheckCpuLevel();
+		M68000_CheckCpuSettings();
 	}
 }
 
@@ -378,7 +562,16 @@ int TOS_LoadImage(void)
 	TosAddress = SDL_SwapBE32(*(Uint32 *)&pTosFile[8]);
 
 	/* Check for reasonable TOS version: */
-	if (TosVersion<0x100 || TosVersion>=0x500 || TosSize>1024*1024L
+	if (TosVersion == 0x000 && TosSize == 16384)
+	{
+		/* TOS 0.00 was a very early boot loader ROM which could only
+		 * execute a boot sector from floppy disk, which was used in
+		 * the very early STs before a full TOS was available in ROM.
+		 * It's not very useful nowadays, but we support it here, too,
+		 * just for fun. */
+		TosAddress = 0xfc0000;
+	}
+	else if (TosVersion<0x100 || TosVersion>=0x500 || TosSize>1024*1024L
 	    || (!bRamTosImage && TosAddress!=0xe00000 && TosAddress!=0xfc0000))
 	{
 		Log_AlertDlg(LOG_FATAL, "Your TOS image seems not to be a valid TOS ROM file!\n"
@@ -417,7 +610,7 @@ int TOS_LoadImage(void)
 	if (TosVersion == 0x0100 && bUseVDIRes)
 	{
 		/* Warn user */
-		Log_AlertDlg(LOG_ERROR, "To use GEM extended resolutions, you must select a TOS >= 1.02.");
+		Log_AlertDlg(LOG_ERROR, "To use extended VDI resolutions, you must select a TOS >= 1.02.");
 		/* And select non VDI */
 		bUseVDIRes = ConfigureParams.Screen.bUseExtVdiResolutions = false;
 	}
@@ -433,6 +626,7 @@ int TOS_LoadImage(void)
 	free(pTosFile);
 
 	bTosImageLoaded = true;
+	TOS_CreateAutoInf();
 
 	return 0;
 }
